############################################################################
#                                                                          #
#             copyright (c) 2004, 2006 ITB, Humboldt-University Berlin     #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""EndRenderer (EndNotes) class"""

# Python stuff
import os, types

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# CMF stuff
from Products.CMFCore.utils import getToolByName

# Bibliography stuff
from Products.CMFBibliographyAT.tool.renderers.base \
     import IBibliographyRenderer, BibliographyRenderer

from Products.CMFBibliographyAT.utils import _default_encoding

class EndRenderer(BibliographyRenderer):
    """
    A specific EndNote renderer that exports bibliographical references
    in the EndNote text format.
    """
    # depends on the BibTeX renderer

    __implements__ = (IBibliographyRenderer ,)

    meta_type = "EndNote Renderer"

    format = {'name':'EndNote',
              'extension':'end'}

    def __init__(self,
                 id = 'endnote',
                 title = "transforms bibtex renderer output"):
        """
        initializes id and title
        """
        self.id = id
        self.title = title

    def isAvailable(self):
        """ test if transforming, from 'BibTex' to 'Endnote' is possible...
        """
        bib_tool = getToolByName(self, 'portal_bibliography') 
        bibtex_renderer = bib_tool.Renderers.bibtex
        return bib_tool.isTransformable('bib', 'end') and \
               bibtex_renderer.isAvailable()
                                                    
    def render(self, objects, output_encoding=None, msdos_eol_style=False, **kwargs):
        """
        renders a bibliography object in EndNote's text format
        """
        if type(objects) not in (types.ListType, types.TupleType):
            objects = [objects]
        bib_tool = getToolByName(objects[0], 'portal_bibliography')
        source = bib_tool.render(objects, 'bib', output_encoding=_default_encoding, msdos_eol_style=msdos_eol_style)
        return self._convertToOutputEncoding(\
            bib_tool.transform(source, 'bib', 'end'),
            output_encoding=output_encoding)

InitializeClass(EndRenderer)


def manage_addEndRenderer(self, REQUEST=None):
    """ """
    try:
        self._setObject('endnote', EndRenderer())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The renderer you attempted to add already exists.',
            action='manage_main')
    return self.manage_main(self, REQUEST)



## Script (Python) "getFieldIndexes"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
return context.portal_bibliography.getSortIndexes()



from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase

# Make sure the dummy types are registered
from Products.CMFBibliographyAT.tests import dummy


ZopeTestCase.installProduct('CMFBibliographyAT')
PRODUCTS = ['CMFBibliographyAT',]
PloneTestCase.setupPloneSite(products=PRODUCTS)


# This is so we can find our own files
from Products.CMFBibliographyAT.tests import GLOBALS
from Globals import package_home
PACKAGE_HOME = package_home(GLOBALS)

from os.path import join
MEDLINE_TEST_MED = join(PACKAGE_HOME, 'medline_test.med')
MEDLINE_TEST_BIB = join(PACKAGE_HOME, 'medline_test.bib')
MEDLINE_TEST_XML = join(PACKAGE_HOME, 'medline_test.xml')
BIBTEX_TEST_BIB = join(PACKAGE_HOME, 'bibtex_test.bib')
IDCOOKING_TEST_BIB = join(PACKAGE_HOME, 'idcooking_test.bib')
PDFFOLDER_TEST_BIB = join(PACKAGE_HOME, 'pdffolder_test.bib')
CMFBAT_TEST_PDF_1 = join(PACKAGE_HOME, 'cmfbat-pdffile1.pdf')
CMFBAT_TEST_PDF_2 = join(PACKAGE_HOME, 'cmfbat-pdffile2.pdf')
CMFBAT_TEST_PDF_3 = join(PACKAGE_HOME, 'cmfbat-pdffile3.pdf')
BIBTEX_TEST_BIB_DUP = join(PACKAGE_HOME, 'bibtex_test_duplicates.bib')
BIBTEX_TEST_MULTI_AUTHORS = join(PACKAGE_HOME,
                                 'bibtex_test_multiauthors.bib')
BIBTEX_TEST_INBOOKREFERENCES = join(PACKAGE_HOME,
                                 'bibtex_test_inbookreferences.bib')
BIBTEX_TEST_LASTFIELDKOMMA = join(PACKAGE_HOME,
                                 'bibtex_test_lastfieldkomma.bib')
BIBTEX_TEST_TYPEFIELD = join(PACKAGE_HOME,
                                 'bibtex_test_typefield.bib')
BIBTEX_TEST_CITE_KEY = join(PACKAGE_HOME,
                            'bibtex_test_cite_key.bib')
CITATION_MANAGER_SOURCE = join(PACKAGE_HOME, 'citationmanager_test.txt')
RIS_SOURCE = join(PACKAGE_HOME, 'ris_test.ris')



##########################################################################
#                                                                        #
#           copyright (c) 2003,2005 ITB, Humboldt-University Berlin      #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
# modified by LOGILAB S.A. (Paris, FRANCE) - 2005                        #
# http://www.logilab.fr/ -- mailto:contact@logilab.fr                    #
##########################################################################

"""BibliographyFolder main class"""

__revision__ = '$Id:  $'


import string, sys, re, os.path, copy
from types import StringType
from operator import and_, add
from md5 import md5

from OFS.Folder import Folder
from Globals import InitializeClass, MessageDialog, PersistentMapping
from DocumentTemplate import sequence

from AccessControl import ClassSecurityInfo
from Acquisition import Acquirer
from Products.CMFCore.permissions import View, ManagePortal,\
     ModifyPortalContent, AddPortalContent, ManageProperties
from Products.CMFCore.utils import getToolByName, _checkPermission
from Products.Archetypes.utils import shasattr
from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import *
else:
    from Products.Archetypes.public import *

from Products.ATContentTypes.content.base import ATCTOrderedFolder
from Products.ATContentTypes.content.folder import ATBTreeFolder
from Products.ATContentTypes.content.folder import ATBTreeFolderSchema
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from Products.CMFBibliographyAT.config import REFERENCE_TYPES, PROJECTNAME, \
     FOLDER_TYPES
from Products.CMFBibliographyAT.interfaces import IBibliographyExport
from Products.CMFBibliographyAT.marshall import BibtexMarshaller

from Products.CMFBibliographyAT.tool.parsers.base import EntryParseError
from Products.CMFBibliographyAT.DuplicatesCriteria import DuplicatesCriteriaManager
from Products.CMFBibliographyAT.utils import log

from Products.CMFBibliographyAT.utils import _encode, _decode

BibFolderIdCookingSchema = Schema((
    StringField('referenceIdCookingMethod',
        searchable = 0,
	default = None,
	default_method = 'getSiteDefaultIdCookerId',
        vocabulary="listEnabledIdCookers",
        widget=SelectionWidget(
	    label="ID Cooking Method for Bibliographical Entries",
            description="Specify the algorithm that shall be used for bibliographical entry ID cooking in this bibliography folder. If not sure what to say here, leave the default.",
            label_msgid="label_bibfolder_idcooking",
            description_msgid="help_bibfolder_idcooking",      
            condition="python:len(object.portal_bibliography.listIdCookers()) > 1",
	    format="select",
	    i18n_domain="cmfbibliographyat",
        ),
    ),
    BooleanField('cookIdsOnBibRefCreation',
        searchable = 0,
	default = True,
	default_method = 'getSiteDefaultCookIdsOnBibRefCreation',
        widget=BooleanWidget(
	    label="ID Cooking on Creation of a Bibliographical Entry",
            label_msgid="label_bibfolder_cookidsoncreation",
            description="Cook IDs for bibliographical entries when created TTW. For this to work, leave new IDs (auto-generated by Plone - if shown at all) untouched on creation of a bibliographical entry. If you change the entry's ID on creation, no ID cooking will be performed. If not sure what to say here, leave the default.",
            description_msgid="help_bibfolder_cookidsoncreation",      
	    i18n_domain="cmfbibliographyat",
        ),
    ),
    BooleanField('cookIdsAfterBibRefEdit',
        searchable = 0,
	default = True,
	default_method = 'getSiteDefaultCookIdsAfterBibRefEdit',
        widget=BooleanWidget(
	    label="ID Re-Cooking after a Bibliographical Entry has been Edited",
            label_msgid="label_bibfolder_cookidsonedit",
            description="Re-cook IDs for bibliographical items after each edit action. This feature is useful, if you want to make sure, that bibliographical reference IDs always stay in sync with the entry's properties (author names, publication year etc. - depending on what properties are used for ID cooking). If not sure what to say here, leave the default.",
            description_msgid="help_bibfolder_cookidsonedit",      
	    i18n_domain="cmfbibliographyat",
        ),
    ),
))

BibFolderBibrefImportSchema = Schema((
    BooleanField('useParserIdsOnImport',
        searchable = 0,
	default = True,
	default_method = 'getSiteDefaultUseParserIdsOnImport',
        widget=BooleanWidget(
	    label="Use IDs from Import Files on Reference Import",
            label_msgid="label_bibfolder_usepidsonimport",
            description="On bibliography import some import formats already offer unique IDs for bibliographical entries. If you want to keep those IDs, tick here. If not sure what to say here, leave the default.",
            description_msgid="help_bibfolder_usepidsonimport",      
	    i18n_domain="cmfbibliographyat",
        ),
    ),
))

BibFolderPdfManagerSchema = Schema((
    LinesField('allowPdfUploadForTypes',
        searchable = 0,
	default = None,
	default_method = 'getSiteDefaultAllowPdfUploadForTypes',
        vocabulary="listReferenceTypes",
        widget=InAndOutWidget(
	    label="Allow PDF Upload Only for Selected Types",
            label_msgid="label_bibfolder_allowpdfuploadfortypes",
            description="Bibliographical reference types of this bibliography folder will only support upload of publications in PDF format if selected here.",
            description_msgid="help_bibfolder_allowpdfuploadfortypes",      
            condition="python:object.portal_bibliography.allowPdfUploadPortalPolicy()",
	    i18n_domain="cmfbibliographyat",
        ),
    ),
    BooleanField('synchronizePdfFileAttributes',
        searchable = 0,
	default = True,
	default_method = 'getSiteDefaultSynchronizePdfFileAttributes',
        widget=BooleanWidget(
	    label="Synchronize PDF File Attributes",
            label_msgid="label_bibfolder_synchronizepdffileattributes",
            description="Synchronize some attributes of an uploaded PDF file with the attribute values of the referring bibliographical entry (currently supported: ID, Roles, Creator).",
            description_msgid="help_bibfolder_synchronizepdffileattributes",      
            condition="python:object.portal_bibliography.allowPdfUploadPortalPolicy()",
	    i18n_domain="cmfbibliographyat",
        ),
    ),
    
))

BibFolderDuplicatesManagerSchema = Schema((
    BooleanField('enableDuplicatesManager',
        searchable = 0,
	default = True,
	default_method = 'getSiteDefaultEnableDuplicatesManager',
        widget=BooleanWidget(
	    label="Enable Duplicates Management",
            label_msgid="label_bibfolder_enableduplicatemanager",
            description="Enable the Duplicates Management engine of this bibliography folder. If enabled (recommended) duplicate bibliographical references will be treated specially on import, cut, copy and paste.",
            description_msgid="help_bibfolder_enableduplicatemanager",      
	    i18n_domain="cmfbibliographyat",
        ),
    ),
    StringField('duplicatesMatchingPolicy',
        searchable = 0,
	vocabulary = 'listDuplicatesMatchingPolicies',
	default = 'local',
        widget=SelectionWidget(
	    label="Duplicates Matching Policy",
            label_msgid="label_bibfolder_duplicatesmatchingpolicy",
            description="If duplicates management is enabled, this value will define the search span for matching objects.",
            description_msgid="help_bibfolder_duplicatesmatchingpolicy",      
	    i18n_domain="cmfbibliographyat",
        ),
    ),
    
))

BibFolderSchema = ATBTreeFolderSchema.copy() + \
                  BibFolderIdCookingSchema.copy() + \
		  BibFolderBibrefImportSchema.copy() + \
		  BibFolderDuplicatesManagerSchema.copy() + \
		  BibFolderPdfManagerSchema.copy() + Schema((		  
    BooleanField('searchable',
        searchable = 0,
	default = True,
	default_method = 'getSiteDefaultSearchableBibFolders',
        widget=BooleanWidget(
	    label="Searchable Bibliography Folder",
            label_msgid="label_bibfolder_searchable_bibfolder",
            description="Only if this option is enabled bibliographical entries in this bibliography folder will be found by site users using the bibliography search form. You can also use this option to hide this bibliography folder's bibliographical entries from &raquo;(Smart) Bibliography Lists&laquo; (if installed).",
            description_msgid="help_bibfolder_searchable_bibfolder",      
	    i18n_domain="cmfbibliographyat",
        ),
    ),
    TextField('intro',
	searchable = 1,
	default = '',
	default_content_type = 'text/html',
	default_output_type = 'text/x-html-captioned', 
	allowed_content_types = ('text/html','text/plain',), 
	widget=RichWidget(
	    label="Intro Text",
    	    label_msgid="label_bibfolder_intro",
    	    description="Adding an introductory text at the  top of the folder listing.",
    	    description_msgid="help_bibfolder_intro",      
    	    condition="python:object.portal_bibliography.allow_folder_intro",
	    rows=15,
	    i18n_domain="cmfbibliographyat",
    	),
    ),
))
BibFolderSchema.moveField('searchable', after='description')

DuplicatesBibFolderSchema = ATBTreeFolderSchema.copy()
finalizeATCTSchema(BibFolderSchema, folderish=True, moveDiscussion=False)

def associateToBibliographyFolder(self, bibfolder=None):
    """ on post-create associate PDF Folder / Duplicates Bibliography Folder to its Bibliography Folder
    """
    if bibfolder is not None:
        if isinstance(bibfolder, basestring) and (at_tool.lookupObject(bibfolder) and True or False):
            bibfolder = at_tool.lookupObject(bibfolder)
        if shasattr(bibfolder, 'portal_type') and (bibfolder.portal_type in FOLDER_TYPES):
            self._assoc_bibliography_folder = bibfolder.UID()

        if not self._assoc_bibliography_folder is not None:
            raise ValueError, 'While associating %s at %s to its Bibliography Folder something went terribly wrong ...' % (self.Title(), self.absolute_url())
													
class BaseBibliographyFolder(Acquirer):
    """
    base class for containers for bibliographic references
    """

    security = ClassSecurityInfo()
    
    filter_content_types = 1
    allowed_content_types = REFERENCE_TYPES
    content_icon = 'bibliography_bibfolder.png'
    
    default_view = 'base_view'
    view_methods = ('folder_listing',
                    'folder_tabular_view',
                    'base_view',
                    )    

    author_urls = {}

    schema = BibFolderSchema
    _at_rename_after_creation = True
    __implements__ = (IBibliographyExport,
                     )
    actions = (
        {
        'id'          : 'local_roles',
        'name'        : 'Sharing',
        'action'      : 'string:${object_url}/folder_localrole_form',
        'permissions' : (ManageProperties,),
        'condition'   : 'python: object.portal_membership.checkPermission("Manage Properties", object)',
        },
    )


    security.declareProtected(View, 'isTranslatable')
    def isTranslatable(self):
        
        bib_tool = getToolByName(self, 'portal_bibliography')
        plone_utils = getToolByName(self, 'plone_utils')
        return bib_tool.isBibFolderTranslatable() and plone_utils.isTranslatable(self)

    security.declareProtected(View, 'getSiteDefaultAllowPdfUploadForTypes')
    def getSiteDefaultAllowPdfUploadForTypes(self):
        """get default values for allowed PDF upload from bibliography tool
        """
        bib_tool = getToolByName(self, 'portal_bibliography')
        return bib_tool.allowPdfUploadForTypes()

    security.declareProtected(View, 'getSiteDefaultSearchableBibFolders')
    def getSiteDefaultSearchableBibFolders(self):
        """get default value for searchable field of bibfolders
        """
        bib_tool = getToolByName(self, 'portal_bibliography')
        return bib_tool.searchableBibFolders()

    security.declareProtected(View, 'ListReferenceTypes')
    def listReferenceTypes(self):
        """list available reference types for use in schema field
        """
        bib_tool = getToolByName(self, 'portal_bibliography')
        return DisplayList(tuple([ (ref_type, ref_type) for ref_type in bib_tool.getReferenceTypes() ]))

    security.declareProtected(View, 'getBibReferences')
    def getBibReferences(self, lazy=False, sort_on='publication_year'):
        """Returns all of the contained bib references, as catalog brains.
        """
        bibtool = getToolByName(self, 'portal_bibliography')
        ref_types = bibtool.getReferenceTypes()
        catalog = getToolByName(self, 'portal_catalog')
        refs = catalog(
            portal_type=ref_types,
            path={'query' : '/'.join(self.getPhysicalPath()),
                  'level' : 0,
		  'depth' : 1,},
            sort_on=sort_on,
            sort_order='reverse',
        )
        if lazy:
            return refs
        return [brain.getObject() for brain in refs]

    ## play nice with FTP and WebDAV

    security.declareProtected(View, 'manage_FTPget')
    def manage_FTPget(self):
        """render all references as one BibTeX file"""
        bibtool = getToolByName(self, 'portal_bibliography')
        return bibtool.render(self, 'bib')

    security.declareProtected(View, 'content_type')
    def content_type(self):
        """
        rely on our default rendering 'applicatio/x-bibtex'
        """
        return 'application/x-bibtex'

    security.declareProtected(View, 'get_size')
    def get_size(self):
        """ The number of objects in this folder """
        return len(self.objectIds())

    security.declareProtected(ModifyPortalContent, 'PUT_factory')
    def PUT_factory(self, name, typ, body):
        """
        Handle HTTP and FTP PUT requests

        What we need to do here is to return something that later
        can be called 'PUT' upon as we are in a bibfolder already
        Also temporarily allow nesting of bibfodlers
        """
        # temporarily allow bibfolders within bibfolders
        types_tool = getToolByName(self, 'portal_types')
        for ptype in ['BibliographyFolder','LargeBibliographyFolder']:
            fti = types_tool[ptype]
            fti.filter_content_types = 0
        return BibliographyFolder(name)  
        

    security.declareProtected(ModifyPortalContent, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP and FTP PUT requests """
        raw = REQUEST.get('BODY')
        id = self.getId()
        parent = self.aq_inner.aq_parent
        if IBibliographyExport in parent.__implements__:
            target_folder = parent
            try:
                target_folder.manage_delObjects([id])
            except AttributeError:
                pass
            # don't forget to impose our restriction again
            types_tool = getToolByName(self, 'portal_types')
            for ptype in ['BibliographyFolder','LargeBibliographyFolder']:
                fti = types_tool[ptype]
                fti.filter_content_types = 1
        else:
            target_folder = self
            target_folder.setTitle(id)
        target_folder.processImport(raw, id)
        RESPONSE.setStatus(204)
        target_folder.reindexObject()
        return RESPONSE


class BaseBibliographyAuthorUrlManager(Acquirer):
    """ enable management of an {author:authorurl} dictionary
    """
    security = ClassSecurityInfo()
    actions = ()
    
    security.declareProtected(View, 'AuthorURLs')
    def AuthorURLs(self):
        """ accessor """
        return self.author_urls

    security.declareProtected(View, 'AuthorUrlList')
    def AuthorUrlList(self):
        """ list of dictionaries for editing """
        link_list = []
        keys = self.AuthorURLs().keys()
        keys.sort()
        for key in keys:
            entry = {'key':key,
                     'value':self.AuthorURLs()[key],
                     }
            link_list.append(entry)
        return link_list
    
    security.declareProtected(ModifyPortalContent, 'setAuthorURLs')
    def setAuthorURLs(self, author_dict):
        """ mutator (entire dictionary) """
        self.author_urls = author_dict
        
    security.declareProtected(ModifyPortalContent, 'addAuthorURLs')
    def addAuthorURL(self, key='dummy', value=None):
        """ new entry """
        self.author_urls[key] = value

    security.declareProtected(ModifyPortalContent, 'processAuthorUrlList')
    def processAuthorUrlList(self, link_list):
        """ creates the links dictionary from the list entries """

        link_dict = {}
        for link in link_list:
            link_dict[link['key']] = link['value']
        self.setAuthorURLs(link_dict)

    # enable look-up by authors
    security.declareProtected(View, 'getPublicationsByAuthors')
    def getPublicationsByAuthors(self, search_list, and_flag = 0):
        """
        returns a filtered list of content values matching
        the publications of the specified authors

        authors MUST be specified by first initial plus lastname
        like in 'J Foo' or ['J Foo', 'J Bar'] 
        """
        if type(search_list) == StringType:
            search_list = [search_list]

        result = []
        bib_tool = getToolByName(self, 'portal_bibliography')
        ref_types = bib_tool.getReferenceTypes()
        filter = {'portal_type' : ref_types}
        for value in self.contentValues(filter=filter):
            author_list = []
            for author in value.getAuthorList():
                entry = author.get('firstname', ' ')[0] \
                        + ' ' \
                        + author.get('lastname', '')
                author_list.append(entry.strip())

            for author in search_list:
                if author in author_list and value not in result:
                    result.append(value)

            if and_flag:
                if value in result:
                    for author in search_list:
                        if author not in author_list:
                            result.remove(value)
        return result

class BaseBibliographyIdCookerManager(Acquirer):
    """ methods that relate to automated ID cooking of bibliographical reference items
    """
    security = ClassSecurityInfo()
    actions = ()
    schema = BibFolderIdCookingSchema
    
    security.declarePublic(View, 'getSiteDefaultIdCookerId')
    def getSiteDefaultIdCookerId(self):
    
	bib_tool = getToolByName(self, 'portal_bibliography')
	return bib_tool.getDefaultIdCooker(with_disabled=False).getId()

    security.declarePublic(View, 'getSiteDefaultUseParserIdsOnImport')
    def getSiteDefaultUseParserIdsOnImport(self):
    
	bib_tool = getToolByName(self, 'portal_bibliography')
	return bib_tool.useParserIdsOnImport()

    security.declarePublic(View, 'getSiteDefaultCookIdsOnBibRefCreation')
    def getSiteDefaultCookIdsOnBibRefCreation(self):
    
	bib_tool = getToolByName(self, 'portal_bibliography')
	return bib_tool.cookIdsOnBibRefCreation()

    security.declarePublic(View, 'getSiteDefaultCookIdsAfterBibRefEdit')
    def getSiteDefaultCookIdsAfterBibRefEdit(self):
    
	bib_tool = getToolByName(self, 'portal_bibliography')
	return bib_tool.cookIdsAfterBibRefEdit()

    security.declarePublic(View, 'listEnabledIdCookers')
    def listEnabledIdCookers(self):
    
	bib_tool = getToolByName(self, 'portal_bibliography')
	return DisplayList(tuple([(bib_tool.getIdCooker(idcooker_id=cooker_id).getId(), bib_tool.getIdCooker(idcooker_id=cooker_id).Title()) for cooker_id in bib_tool.listIdCookers(with_disabled=False) ]))

    security.declarePublic(AddPortalContent, 'listEnabledIdCookers')
    def getIdCooker(self):
    
	bib_tool = getToolByName(self, 'portal_bibliography')
	return bib_tool.getIdCooker(idcooker_id=self.getReferenceIdCookingMethod(), with_disabled=True)

    security.declareProtected(AddPortalContent, 'cookId')
    def cookId(self, ref):
        """
        :param ref: a single bibliography entry
        """
        # invalid entry code : nobody1000
        # if entry has no author and no publication_year - invalid entry
        
        newid = ''
        bib_tool = getToolByName(self, 'portal_bibliography')
	if type(ref) != type({}):
            return "nobody1000"

	newid = bib_tool.cookReferenceId(ref, idcooker_id=self.getReferenceIdCookingMethod(), use_pid_on_import=self.getUseParserIdsOnImport(), with_disabled=True)
	    	    
	while self.hasObject(newid):
            newid = self.nextId(newid)
        return newid

    def nextId(self, testid):
        bib_tool = getToolByName(self, 'portal_bibliography')
	return bib_tool.nextId(testid)


class BaseBibliographyPublicationRanking(Acquirer):
    """ method that deal with publication ranking
    """
    security = ClassSecurityInfo()
    actions = (
        {
        'id'           : 'defaults',
        'name'         : 'Defaults',
        'action'       : 'string:${object_url}/bibliography_defaultsForm',
        'permissions'  : (ModifyPortalContent,),
        'condition'   : 'python: object.portal_membership.checkPermission("Modify portal content", object)',
        },
    )
    
    security.declareProtected(View, 'Top')
    def Top(self, number=None, order=None, explicit=0):
        """
        Returns all ranked entries in order of their ranking.
        If number is set, returns the top 'number' reference objects
        (or all if 'number' is greater than the number of ranked entries).
        If 'explicit' is set, only returns the explicitly ranked
        entries, otherwise the default ordering is used
        """
        if number:
            number = int(number) 
        top_elements = []
        entries = self.listFolderContents()
        for entry in entries:
            if getattr(entry, 'rank', None):
                top_elements.append((entry.rank, entry))
        top_elements.sort()
        result = [entry[1] for entry in top_elements]

        if result and number:
            return result[:number]
        elif result:
            return result

        if not explicit:
            if number:
                return self.defaultOrder(order)[:number]
            else:
                return self.defaultOrder(order)
        return []

    def defaultOrder(self, order=None):
        """
        The entries in default ordering:
        If no orer is specified,
        (('publication_year', 'cmp', 'desc'),('Authors', 'nocase', 'asc'))
        is used.
        Order must be formated to match the
        'DocumentTemplate.sequence' syntax
        """
        raw_list = self.listFolderContents()

        if order is None:
            sort_on = (('publication_year', 'cmp', 'desc'),
                       ('Authors', 'nocase', 'asc'))
        else:
            sort_on = order

        return sequence.sort(raw_list, sort_on)

    security.declareProtected(ModifyPortalContent, 'setTop')
    def setTop(self, ids=[]):
        """ sets the publication ranking """
        rank = 0
        self._resetRanking()

        for obj_id in ids:
            obj = getattr(self, obj_id, None)
            if obj:
                rank += 1
                obj.rank = rank

    def _resetRanking(self):
        """ resets the ranking of reference objects """
        for obj in self.contentValues():
            obj.rank = None


class BaseBibliographyImportManager(Acquirer):
    """ management for bibliography import
    """
    security = ClassSecurityInfo()

    schema = BibFolderBibrefImportSchema
    actions = (
        {
        'id'           : 'import',
        'name'         : 'Import',
        'action'       : 'string:${object_url}/bibliography_importForm',
        'permissions'  : (AddPortalContent,),
        'condition'    : 'python: object.portal_membership.checkPermission("Add portal content", object)',
	'category'     : 'document_actions',
        },
    )
    
    security.declareProtected(AddPortalContent, 'logImportReport')
    def logImportReport(self, report):
        """Store the import report.
        """
        # finish building and write the report
        old_report = self.getProperty('import_report', '')
	report = report + '='*30 + '\n' + _encode(_decode(old_report))
        self.manage_changeProperties(import_report=report)

    def initReport(self, origin):
        # start building the report
        mtool  = getToolByName(self, 'portal_membership')
        member = mtool.getAuthenticatedMember()
        member_id = member.getId()
        fullname = member.getProperty('fullname', None)
        if not fullname:
            fullname = 'unknown fullname'
        import_date = self.ZopeTime()
        
        tmp_report = '[%s] Imported by %s (%s) from %s:\n\n' \
                     %(import_date, member_id, fullname, origin)
        return tmp_report

    def wrapupReport(self, tmp_report):
        # finish building and write the report
        old_report = self.import_report
        report = tmp_report \
               + '='*30 + '\n' \
               + old_report
        self.manage_changeProperties(import_report=report)

    security.declareProtected(ModifyPortalContent, 'buildReportLine')
    def buildReportLine(self, import_status, entry, url=None, relations=None):
        """ format a line to be added in the import report """
        if entry.has_key('authors'):
            authors = entry.get('authors')
            authors_list = []
            for author in authors:
                firstname = author.get('firstname')
                middlename = author.get('middlename', '')
                lastname = author.get('lastname')
                authors_list.append('%s %s %s' %(firstname,
                                                 middlename,
                                                 lastname))
            ref_authors = ', '.join(authors_list)
        else:
            ref_authors = 'Anonymous'

        ref_authors = _decode(ref_authors)
        ref_title = _decode(entry.get('title'))
        for car in ['\n', '\r', '\t', '  ', '  ']:
            ref_title = ref_title.replace(car,'')
        line = u'%s - %s' % (ref_authors, ref_title)

        if entry.has_key('publication_year'):
            line = u'%s (%s)' %(line, entry.get('publication_year'))

        if import_status == 'OK':
            line = u'Successfully Imported: %s' % line
        if import_status == 'ok' and relations:
            relations = _decode(relations)
            line = u'%s (Inferred author references: %s)' \
                   % (line, relations)
        if import_status == 'ok' and url:
            line = u'%s [<a href="%s">view</a>], ' \
                   u'[<a href="%s">edit</a>]' \
                   % (line, url, url+'/edit')
        else:
            line = u'%s: %s' % (import_status, line)
        return _encode(line + '.\n')

    security.declareProtected(AddPortalContent, 'processSingleImport')
    def processSingleImport(self, entry, span_of_search=None, force_to_duplicates=False,
                            skip_matching=False, infer_references=True):
        """ called for importing a single entry

        :type entry: dict
        :param entry: a single bibliography entry
        :type skip_matching: Boolean
        :param skip_matching: toogle to skip matching objects
                              (used for force import)
        :type force_to_uplicates: Boolean
        :param force_to_duplicates: used for migration

        returns a tuple description of success of operation
                (desc, code, obj)
                eg. ('Failed : this entry', 'FAILED', obj)

                possible codes: OK, FAILED, (+ duplicate)
        """
        url = None
        relations = None
        obj = None
        
        if isinstance(entry, EntryParseError):
            return ('Failed: %s\n' % entry.description, 'FAILED')
        
	span_of_search = span_of_search or self.getDuplicatesMatchingPolicy()
	
	# import entry as bibref object first
        try:
	    newid = self.cookId(entry)
            if newid and newid != "nobody1000":
                type = entry.get('reference_type', 'ArticleReference')
		del entry['reference_type']
                if type in self.allowed_content_types:
                    self.invokeFactory(type, newid)
                    obj = getattr(self, newid)
                    obj.edit(**entry)
                    url = obj.absolute_url()
                    if obj.showMemberAuthors() \
                           or obj.getMember_publication_authors() \
                           and infer_references:
                        relations = obj.inferAuthorReferences()

    	    if self.getEnableDuplicatesManager():
		if not skip_matching or force_to_duplicates:
        	    test, matched_objects = self.isDuplicate(obj, span_of_search)
		    if test or force_to_duplicates:

			# in any case, we want the duplicate obj to be aware of local AND global matches
			if span_of_search != 'global':
			    dummy, matched_objects = self.isDuplicate(obj, 'global')

            		duplicate = self.moveToDuplicatesFolder(obj, matched_objects)
            		return ('Skipped: %s\n' % obj.Title() or 'no info',
                	        'SKIPPED', duplicate)
			    
        except: # for debugging
            # Remove the \n from import_status so that it all appears
            # on one line in the import_report, which keeps the count
            # the same as that reported.
            import_status = "Error type: %s. Error value: %s" \
                          % (sys.exc_info()[0],
                             sys.exc_info()[1])
        
        import_status = 'ok'
        
	report_line = self.buildReportLine(import_status,
                                           entry,
                                           url=url,
                                           relations=relations)
        return (report_line, import_status, obj)


    security.declareProtected(AddPortalContent, 'processImport')
    def processImport(self, source, filename, format=None, return_obs=False):
        """
        main routine to be called for importing entire files
        from custom code.
        """
        current_report = self.initReport('file %s' % filename)

        # get parsed entries from the Bibliography Tool
        bib_tool = getToolByName(self, 'portal_bibliography')
        entries = bib_tool.getEntries(source, format, filename)

        obs = []
        for entry in entries:
            if entry.get('title'):
                if return_obs:
                    upload = self.processSingleImport(entry, infer_references=bib_tool.inferAuthorReferencesAfterImport())
                    obs.append(upload[2])
                else:
                    upload = self.processSingleImport(entry, infer_references=bib_tool.inferAuthorReferencesAfterImport())
                current_report = current_report + upload[0]

        self.logImportReport(current_report)        
	
	# we have to set a transaction savepoint here
	bib_tool.transaction_savepoint(optimistic=True)

        if return_obs:
            return obs


class BaseBibliographyDuplicatesManager(Acquirer):
    """ duplication handling for bibliography folders
    """
    security = ClassSecurityInfo()
    actions = (
	{
        'id'           : 'duplicates',
        'name'         : 'Manage Duplicates',
        'action'       : 'string:${object_url}/bibliography_duplicatesForm',
        'permissions'  : (ModifyPortalContent,),
        'condition'    : 'python: object.getEnableDuplicatesManager() and object.portal_membership.checkPermission("Modify portal content", object)',
        },
    )

    _assoc_duplicates_folder = None
    schema = BibFolderDuplicatesManagerSchema

    security.declareProtected(View, 'getDuplicatesFolder')
    def getDuplicatesFolder(self, id = 'duplicates'):
        """
        Returns a bibliography folder similar container within this 
	bibliography folder for storing duplicate bibliographical entries. 
	This method creates the folder if needed.
        """
	at_tool = getToolByName(self, 'archetype_tool')
	
        if self._assoc_duplicates_folder is None:
            tt = getToolByName(self, 'portal_types')
            fti = tt[self.portal_type]
            allowed_types = fti.allowed_content_types
	    duplicates_folder_portal_type = 'DuplicatesBibliographyFolder'
            fti.allowed_content_types += (duplicates_folder_portal_type,)
            self.invokeFactory(type_name = duplicates_folder_portal_type,
                               id = id,
                               )
            self[id].setTitle('Pending Duplicate Bibliography Entries')
            fti.allowed_content_types = allowed_types
	    self._assoc_duplicates_folder = self[id].UID()
	    self[id]._assoc_bibliography_folder = self.UID()
	
	return at_tool.lookupObject(self._assoc_duplicates_folder)    


    security.declarePublic(View, 'getSiteDefaultSynchronizePdfFileAttributes')
    def getSiteDefaultEnableDuplicatesManager(self):
    
	bib_tool = getToolByName(self, 'portal_bibliography')
	return bib_tool.enableDuplicatesManager()

    def duplicatesExist(self):
        """
        test if duplicates exist (used for action tab)
        """
	duplicates_folder = None
	at_tool = getToolByName(self, 'archetype_tool')
	if self._assoc_duplicates_folder is not None:
	    duplicates_folder = at_tool.lookupObject(self._assoc_duplicates_folder)
	
	if duplicates_folder:
	    return duplicates_folder.contentValues() and True or False
	
	return False    

    def getDuplicatesCriteria(self):
        """
         returns rights criteria depending on if they are
         defined portal or folder wide
        """
	return self.getSelectedCriteria()
    getCriterias = getDuplicatesCriteria

    def listDuplicatesMatchingPolicies(self):
    
        return DisplayList((('local', 'local_duplicates_matchingpolicy'), ('global', 'global_duplicates_matchingpolicy')))

    def isDuplicate(self, bibref_item, span_of_search=None):
        """
        checks if entry is duplicated
        """
        if not self.getEnableDuplicatesManager():
	    return False, []
	    
	span_of_search = span_of_search or self.getDuplicatesMatchingPolicy()    
	all_criteria = self.getDuplicatesCriteria()
        bib_tool = getToolByName(self, 'portal_bibliography')
        entry = bib_tool.getEntryDict(bibref_item)
	ref_types = bib_tool.getReferenceTypes()
        have = all_criteria.has_key 
        global_tests = []
        filter = {'portal_type':ref_types}
        # get search span from criteria
        if span_of_search == 'local':
            # local configuration
            acquired_objects = self.contentValues(filter=filter)
            # print "***Debuging*** found %s" % aquired_objects
        elif span_of_search == 'global':
            acquired_objects = []
            portal_catalog = getToolByName(self, 'portal_catalog')
            all_folders = portal_catalog(meta_type = FOLDER_TYPES)
            for each_result in all_folders:
                acquired_objects += each_result.getObject().contentValues(filter=filter)
        else:
            raise ValueError, "span of search for duplicates has an " + \
                  "invalid value : %s" % span_of_search

	# no string exceptions please
	# raise Exception('span of search for duplicates has an invalid value : %s' % span_of_search)

        for existing_object in [ obj for obj in acquired_objects if obj.UID() != bibref_item.UID() ]:
            bib_type = entry.get('reference_type', 'ArticleReference')
            if not have(bib_type) :
                continue
            criteria = all_criteria[bib_type]
            if not criteria:
                return False, []
            # authors need special treatment
            for attribute in criteria:
                # authors need special treatment
                if attribute == 'authors':
                    equal = self.compareAuthors(entry, existing_object)
                    if not equal:
                        break
                else:
                    x = entry.get(attribute, None)
                    try:
                        get_func = getattr(existing_object,
                                           'get'+ attribute.capitalize())
                    except AttributeError:
                        try:
                            get_func = getattr(existing_object,
                                               attribute.capitalize())
                        except AttributeError:
                            print "can't do ", 'get'+attribute.capitalize(), \
                                  'or',  attribute.capitalize()
                            break
                    y = _decode(get_func())
		    if y and y[-1] == '.': y = y[:-1]
		    if x and x[-1] == '.': x = x[:-1]
		    if x != y:
                        #print "***Debug***: found difference"
                        #print "%s doesn't match %s" % (x, y)
                        break
            else:
                global_tests.append((True, existing_object))
        matching_objects = [ext_obj for (test, ext_obj) in global_tests \
                            if test]
        global_bools = [test for (test, ext_obj) in global_tests]
        if global_tests:
            return reduce(and_, global_bools), matching_objects
        return False, []

    def findDuplicated(self, uid=None):
    
	at_tool = getToolByName(self, 'archetype_tool')
	bib_tool = getToolByName(self, 'portal_bibliography')
	
	if uid and shasattr(self.getDuplicatesFolder(), uid):
	    
	    duplicate = at_tool.lookupObject(uid)
	    test, matched_objects = self.isDuplicate(duplicate, span_of_search='global')
	    if test:
		duplicate.setIs_duplicate_of(matched_objects)	

	    return "Matching bibliographical reference of duplicate item with UID %s updated" % uid
	    
	elif uid is None:
	    for duplicate in self.getDuplicatesFolder().contentValues():
	    
		test, matched_objects = self.isDuplicate(duplicate, span_of_search='global')
		if test:
		    duplicate.setIs_duplicate_of(matched_objects)	


    def skipDuplicated(self, uid):
        """
        skip function on an entry in _duplicates (simple deletion)
        
        :type key: string
        :param key: identifier for the entry stored in _duplicates
        """
	at_tool = getToolByName(self, 'archetype_tool')
	duplicate_bibref_item = at_tool.lookupObject(uid)
	if duplicate_bibref_item:
	    self.getDuplicatesFolder().manage_delObjects([duplicate_bibref_item.getId()])
	    
        else:
            return "UID %s does not exist - cannot delete referenced " + \
                   "duplicate entry" % key
		   
    def compareAuthors(self, entry, existing):
        new_last_names = [_decode(a.get('lastname')) for a in entry.get('authors',[])]
        old_last_names = [_decode(a.get('lastname')) for a in existing.getRawAuthors()]
        if new_last_names == old_last_names:
            return True
        return False

    def moveToDuplicatesFolder(self, bibref_object, matched_objects):
        """
        moves bibref item to associated Duplicates Bibliography Folder for post-processing

        :type entry: dict 
        :param entry: a single bibliography entry
        :type matched_object: AT object
        :param matched_object: the entry matches this object
                               (already present on site) 
        """
	at_tool = getToolByName(self, 'archetype_tool')
	bib_tool = getToolByName(self, 'portal_bibliography')
	if bibref_object.portal_type in bib_tool.getReferenceTypes():
	
            tt = getToolByName(self, 'portal_types')
            fti = tt[self.getDuplicatesFolder().portal_type]
            allowed_types = fti.allowed_content_types
            fti.allowed_content_types += (bibref_object.portal_type,)

	    bib_tool.transaction_savepoint(optimistic=True)
	    bibref_object.setIs_duplicate_of(matched_objects)
	    bibref_uid = bibref_object.UID()
	    
	    objs = self.manage_cutObjects([bibref_object.getId(),])
	    self.getDuplicatesFolder().manage_pasteObjects(objs)
	    
	    bibref_object = at_tool.lookupObject(bibref_uid)
	
            fti.allowed_content_types = allowed_types

	    return bibref_object
	    
	return None    

    def forceDuplicated(self, uid):
        """
        force function on an entry in _duplicates
        (forces import of new entry)
        """
        initReport = self.initReport('duplication management')
	at_tool = getToolByName(self, 'archetype_tool')
	bib_tool = getToolByName(self, 'portal_bibliography')
	
        # process import for duplicated entry
        duplicate_bibref_item = at_tool.lookupObject(uid)

	duplicate_bibref_item.setIs_duplicate_of(None)
        
	duplicates_manager_restore = self.getEnableDuplicatesManager()
	self.setEnableDuplicatesManager(False)
	objs = self.getDuplicatesFolder().manage_cutObjects([duplicate_bibref_item.getId(),])
	self.manage_pasteObjects(objs)
	self.setEnableDuplicatesManager(duplicates_manager_restore)
	
	

	entry = bib_tool.getEntryDict(duplicate_bibref_item)
	url = duplicate_bibref_item.absolute_url()

	report_line = self.buildReportLine('forced',
                                           entry,
                                           url=url,
                                           relations=None)
	
        current_report = initReport + report_line
        self.wrapupReport(current_report)
        return current_report

    def replaceDuplicated(self, uid):
        """
        replace function on an entry in _duplicates
        (replaces originial finding with new entry)
        
        :type key: string
        :param key: identifier for the entry stored in _duplicates
        """
	at_tool = getToolByName(self, 'archetype_tool')
	bib_tool = getToolByName(self, 'portal_bibliography')
	duplicate_bibref_item = at_tool.lookupObject(uid)
	
	entry = bib_tool.getEntryDict(duplicate_bibref_item)
	
	for orig_obj in duplicate_bibref_item.getIs_duplicate_of():
    	    
	    orig_obj.setAuthors(entry.get('authors', []))
            orig_obj.edit(**entry)

	    # hmmm... replace means: also replace the PDF file (if any is associated)
	    if duplicate_bibref_item.getPdf_file():
		orig_obj.setPdf_file(duplicate_bibref_item.getPdf_file())
		
	    # and if id cooking after bibref edit is enabled, we have to re-cook the orig_obj's ID	
	    if self.getCookIdsAfterBibRefEdit():
		orig_obj.bibliography_entry_cookId()	
	
	# nasty trick, isn't it? the bibref item does not know that it sleeps in the duplicates bibliography folder... but we do!!!
	duplicates_folder = duplicate_bibref_item.getBibFolder()
	duplicates_folder.manage_delObjects([duplicate_bibref_item.getId()])

        return 'replace begin implemented'

    def delayDuplicated(self, uid):
        """
        delay function on an entry in _duplicates (does nothing)
        """
        pass

    security.declarePublic('getDuplicates')
    def getDuplicates(self, uid=None):
	"""
	get a dict with uid: bibref_object entries
	"""    
    	if uid: 
	    at_tool = getToolByName(self, 'archetypes_tool')
	    return at_tool.lookupObject(uid)

	bib_tool = getToolByName(self, 'portal_bibliography')
	duplicates_dict = {}
	filter = {'portal_type': bib_tool.getReferenceTypes(), }
        for obj in self.getDuplicatesFolder().contentValues(filter=filter):
	    duplicates_dict[obj.UID()] = obj

	return duplicates_dict    
                    
    security.declareProtected(ModifyPortalContent, 'handleAction')
    def handleAction(self, REQUEST = None):
        """
        run through duplicates and determine what to do according to request
        """
        logs = []
        seq_of_uids = []
        for uid in self.getDuplicates().keys():
            seq_of_uids.append(uid)
        for uid in seq_of_uids:
            if uid in REQUEST.keys():
                action = REQUEST.get(uid)
                action_func = getattr(self, '%sDuplicated' % action)
                log_msg = action_func(uid)
                if log_msg:
                    logs.append(log_msg)
        return ', '.join([ _encode(_decode(log.replace('\n\n',' ').replace('\n',''))) for log in logs ])

    security.declareProtected(ModifyPortalContent, 'rescanBibfolderForDuplicates')
    def updateDuplicatesFolder(self, sort_on='modified', sort_order='reverse', span_of_search=None):
        """
        run through bibliographical entries and check if they are duplicates
        """
	logs = []
	bib_tool = getToolByName(self, 'portal_bibliography')
	ctool = getToolByName(self, 'portal_catalog')
	span_of_search = span_of_search or self.getDuplicatesMatchingPolicy()
	query_bibfolder = {
	    'sort_on': sort_on,
	    'portal_type': bib_tool.getReferenceTypes(),
	    'path': {'depth': 1, 'query': '/'.join(self.getPhysicalPath()), },
	}
	if sort_order:
	    query_bibfolder['sort_order'] = sort_order

	bibfolder_contentValues = [ bibref_item.getObject() for bibref_item in ctool(**query_bibfolder) ]
	
	# walk through the list of duplicates from end to beginning...
	bibfolder_contentValues.reverse()
	
	count = 0
	for bibref_item in bibfolder_contentValues:
	
	    test, matching_objects = self.isDuplicate(bibref_item, span_of_search=span_of_search)
	    if test:
		count += 1
		self.moveToDuplicatesFolder(bibref_item, matching_objects)
	
	return 'Bibliography Duplicate Update: %s bibref items have been regarded as duplicate bibliographical entries.' % count


class BaseBibliographyPdfFolderManager(Acquirer):

    security = ClassSecurityInfo()
    
    schema = BibFolderPdfManagerSchema
    actions = ()
    
    _assoc_pdf_folder = None
    _delete_associated_pdffiles = False
    _move_associated_pdffiles = False
    
    security.declareProtected(View, 'getPdfFolder')
    def getPdfFolder(self, id = 'pdfs'):
        """
        Returns a folder for storing files
        Creates it if needed
        """
	at_tool = getToolByName(self, 'archetype_tool')
	
	if self._assoc_pdf_folder is None:
    	    if not self.hasObject(id):
        	tt = getToolByName(self, 'portal_types')
        	fti = tt[self.portal_type]
        	allowed_types = fti.allowed_content_types
        	fti.allowed_content_types += ('PDF Folder',)
        	self.invokeFactory(type_name = 'PDF Folder',
            	                   id = id,
                	           )
        	self[id].setTitle('PDFs')
        	fti.allowed_content_types = allowed_types
		self._assoc_pdf_folder = self[id].UID()
	pdf_folder = at_tool.lookupObject(self._assoc_pdf_folder)
	
	# test, if the returned object really is the pdf folder:
	if pdf_folder.getPhysicalPath()[:-1] == self.getPhysicalPath():
	    return pdf_folder
	else:
	    # BULLSHIT, reference to PDF folder broken.
	    pdf_folder = eval('self.%s' % id)
	    try: self._assoc_pdf_folder = pdf_folder.UID()
	    except: pass
	    return pdf_folder

    security.declarePublic(View, 'getSiteDefaultSynchronizePdfFileAttributes')
    def getSiteDefaultSynchronizePdfFileAttributes(self):
    
	bib_tool = getToolByName(self, 'portal_bibliography')
	return bib_tool.synchronizePdfFileAttributes()

class BaseBibliographyObjectManager(Acquirer):

    security = ClassSecurityInfo()
    _base_folder_class = Folder
    
    def manage_delObjects(self, ids=[], REQUEST=None):

        self._delete_associated_pdffiles=True
	return self._base_folder_class.manage_delObjects(self, ids=ids, REQUEST=REQUEST)

    def manage_cutObjects(self, ids=[], REQUEST=None):

        self._move_associated_pdffiles=True
	return self._base_folder_class.manage_cutObjects(self, ids=ids, REQUEST=REQUEST)
	
    def manage_renameObject(self, id, new_id, REQUEST=None):
        """ also rename PDF file (if any is associated)
        """
        # rename ids in self first
        self._base_folder_class.manage_renameObject(self, id, new_id, REQUEST=REQUEST)
					
        # then try to rename PDF file
	object = self._getOb(new_id)
        pdf_file = shasattr(object, 'getPdf_file') and object.getPdf_file() or None

        # manage_renameObject is called during setId. thus, whenever setId is called
        # we do not need to explicitly call the pdffile id cooker, this is done here!!!
	if pdf_file and self.getSynchronizePdfFileAttributes():
	    object.bibliography_pdffile_cookId(with_disabled=True)
												    	
    def manage_pasteObjects(self, cb_copy_data=None, REQUEST=None):

	ofs_result = self._base_folder_class.manage_pasteObjects(self, cb_copy_data=cb_copy_data, REQUEST=REQUEST)
	self._move_associated_pdffiles = False

	# re-cook ids of bibref items and (again) of associated PDF files if requested in bibfolder settings
	if self.getCookIdsAfterBibRefEdit():
	    
	    bib_tool = getToolByName(self, 'portal_bibliography')	    
	    result = []
	    for ofs_item in ofs_result:
		bibref_old_id = ofs_item['id']
		bibref_ofstemp_id = ofs_item['new_id']
		item = getattr(self, bibref_ofstemp_id)
		bibref_new_id = bib_tool.cookReferenceId(ref=item, idcooker_id=self.getReferenceIdCookingMethod(), with_disabled=True)
		if bibref_new_id != 'nobody1000':
		    item.bibliography_entry_cookId(with_disabled=True)
		    # no subsequent PDF file ID cooking needed here, 
		    # it will be invoked by manage_renameObject 
		    # during bibliography entry ID cooking
		    result.append({'id': bibref_old_id, 'new_id': item.getId(),})
		else:
		    result.append({'id': bibref_old_id, 'new_id': bibref_ofstemp_id,})

	else: 
	    result = ofs_result

	if self.portal_type in FOLDER_TYPES:
	    
	    # duplicate checking only if we are not yet in the duplicates folder (or wherever)
	    for ofs_item in result:

		new_bibref_item_id = ofs_item['new_id']
		new_bibref_item = self[new_bibref_item_id]
		test, matched_objects = self.isDuplicate(new_bibref_item)
		if test:
		    dummy, matched_objects = self.isDuplicate(new_bibref_item, span_of_search='global')
		    self.moveToDuplicatesFolder(new_bibref_item, matched_objects)

	    # TODO: notify the user that some objects may have been moved to DuplicatesBibliographyFolder...
	        
	return result

class BibliographyFolder(BaseBibliographyIdCookerManager,
			 BaseBibliographyPdfFolderManager, 
			 BaseBibliographyDuplicatesManager,
			 BaseBibliographyImportManager,
			 BaseBibliographyAuthorUrlManager, 
			 BaseBibliographyPublicationRanking,
			 BaseBibliographyObjectManager, 
                         BaseBibliographyFolder, 
                         ATCTOrderedFolder,
                         DuplicatesCriteriaManager):
    """container for bibliographic references
    """
    _base_folder_class = ATCTOrderedFolder

    def __init__(self, id, **kwargs):
        self._duplicates = PersistentMapping()
        self._base_folder_class.__init__(self, id, **kwargs)
        DuplicatesCriteriaManager.__init__(self)


    actions = BaseBibliographyIdCookerManager.actions + \
	      BaseBibliographyPdfFolderManager.actions + \
              BaseBibliographyDuplicatesManager.actions + \
	      BaseBibliographyImportManager.actions + \
	      BaseBibliographyAuthorUrlManager.actions + \
	      BaseBibliographyPublicationRanking.actions + \
              BaseBibliographyFolder.actions

    schema = BibFolderSchema
    archetype_name = "Bibliography Folder"
    
    __implements__ = BaseBibliographyFolder.__implements__ + \
                     ATCTOrderedFolder.__implements__

    import_report = ''
    _properties= ATCTOrderedFolder._properties + \
                 ({'id':'import_report', 'type': 'text', 'mode': 'w'},)

    manage_options = (
        ATCTOrderedFolder.manage_options[:2]
        + DuplicatesCriteriaManager.manage_options
        + ATCTOrderedFolder.manage_options[2:]
        )

								
class LargeBibliographyFolder(BaseBibliographyIdCookerManager,
			      BaseBibliographyPdfFolderManager, 
		    	      BaseBibliographyDuplicatesManager,
			      BaseBibliographyImportManager,
			      BaseBibliographyAuthorUrlManager, 
			      BaseBibliographyPublicationRanking,
   			      BaseBibliographyObjectManager, 
                              BaseBibliographyFolder,
            	              ATBTreeFolder,
                              DuplicatesCriteriaManager,
                              ):
    """container for bibliographic references
    """
    _base_folder_class = ATBTreeFolder
    def __init__(self, id, **kwargs):
        self._duplicates = PersistentMapping()
        self._base_folder_class.__init__(self, id, **kwargs)
        DuplicatesCriteriaManager.__init__(self)

    actions = BaseBibliographyIdCookerManager.actions + \
	      BaseBibliographyPdfFolderManager.actions + \
              BaseBibliographyDuplicatesManager.actions + \
	      BaseBibliographyImportManager.actions + \
	      BaseBibliographyAuthorUrlManager.actions + \
	      BaseBibliographyPublicationRanking.actions + \
              BaseBibliographyFolder.actions
              
    schema = BibFolderSchema
    archetype_name = "Large Bibliography Folder"
    global_allow = 0

    __implements__ = BaseBibliographyFolder.__implements__ + \
                     ATBTreeFolder.__implements__
    
    import_report = ''
    _properties= ATBTreeFolder._properties + \
                 ({'id':'import_report', 'type': 'text', 'mode': 'w'},)

    manage_options = (
        ATBTreeFolder.manage_options[:2]
        + DuplicatesCriteriaManager.manage_options
        + ATBTreeFolder.manage_options[2:]
        )

    def listDAVObjects(self):
        """This may have unpleasant side-effects!  Turn this into a
        generator that deactivates objects after they have been yield-ed.
        """
        for obj in self._tree.itervalues():
            # Wrap the object for security checks and returning...
            obj = obj.__of__(self)
            # Check to see if the object has been changed (elsewhere in the
            # current transaction/request.
            changed = getattr(obj, '_p_changed', False)
            deactivate = not changed
            if _checkPermission(View, obj):
                yield obj
            # Only deactivate (and retrieve memory) if the object hasn't been
            # changed.
            if deactivate:
                obj._p_deactivate()


class DuplicatesBibliographyFolder(BaseBibliographyIdCookerManager,
				   BaseBibliographyPdfFolderManager,
				   BaseBibliographyObjectManager, 
				   BaseBibliographyFolder,
                                   ATCTOrderedFolder):
				       
    """container for duplicates of bibliographic references
    """
    archetype_name = "Duplicates Bibliography Folder"

    security = ClassSecurityInfo()
    
    global_allow = False
    filter_content_types = 1
    allowed_content_types = ()
    content_icon = 'bibliography_duplicatesfolder.png'
    
    default_view = 'base_view'
    view_methods = ('folder_listing',
                    'folder_tabular_view',
                    'base_view',
                    )    

    schema = DuplicatesBibFolderSchema
    
    _at_rename_after_creation = True
    __implements__ = ATCTOrderedFolder.__implements__
    
    _assoc_bibliography_folder = None
    
    # this method is provided as a function above
    associateToBibliographyFolder = associateToBibliographyFolder

    def getBibFolder(self):
	""" returns associated bibliography folder
        """
	at_tool = getToolByName(self, 'archetype_tool')
	return at_tool.lookupObject(self._assoc_bibliography_folder)

    def getPdfFolder(self):
        at_tool = getToolByName(self, 'archetype_tool')
	bibfolder = at_tool.lookupObject(self._assoc_bibliography_folder)
	if bibfolder is not None:
	    return bibfolder.getPdfFolder()

    def getDuplicatesFolder(self):
	return self
	
    def getSynchronizePdfFileAttributes(self, **kwargs):
	return True	
	
    def getCookIdsAfterBibRefEdit(self, **kwargs):
	return True	
	
    def getUseParserIdsOnImport(self, **kwargs):
	return False	

    def getReferenceIdCookingMethod(self, **kwargs):
	return 'uid'	
	
    def manage_beforeDelete(self, item, container):
	""" do some cleaning up before vanishing forever """

	# manage_beforeDelete is deprecated in Zope 2.11+
	bibfolder = self.getBibFolder()
	bibfolder._assoc_duplicates_folder = None
	ATCTOrderedFolder.manage_beforeDelete(self, item, container)	
	
    def manage_renameObject(self, id, new_id, REQUEST=None):
	""" make sure that we can rename bibref item content types """
	tt = getToolByName(self, 'portal_types')
	bib_tool = getToolByName(self, 'portal_bibliography')
	fti = tt[self.portal_type]
	fti.allowed_content_types = tuple(bib_tool.getReferenceTypes())
	
	# BaseBibliographyPdfFolderManager will also call the ATCTOrderedFolder.manage_renameObject method
	# so nothing more to do here...
	BaseBibliographyObjectManager.manage_renameObject(self, id, new_id, REQUEST=REQUEST)
	
	fti.allowed_content_types = ()
	
									
registerType(BibliographyFolder, PROJECTNAME)
registerType(LargeBibliographyFolder, PROJECTNAME)
registerType(DuplicatesBibliographyFolder, PROJECTNAME)



##################################################
#                                                #
#    Copyright (C), 2004, Raphael Ritz           #
#    <r.ritz@biologie.hu-berlin.de>              #
#                                                #
#    Humboldt University Berlin                  #
#                                                #
##################################################

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup
from Products.CMFCore.utils import getToolByName


class TestRenderXML(PloneTestCase.PloneTestCase):
    """Test rendering to XML.
    """

    def afterSetUp(self):
        self._refreshSkinData()

    # some utility methods

    def getEmptyBibFolder(self):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = 'bib_folder')
        return getattr(uf, 'bib_folder')

    def getPopulatedBibFolder(self, source_file=setup.MEDLINE_TEST_MED, format="med"):
        bf = self.getEmptyBibFolder()
        source = open(source_file, 'r').read()
        bf.processImport(source, 'source.%s' % format)
        return bf

    # the individual tests

    def testRenderXML(self):
        bf = self.getPopulatedBibFolder()
        xml_source = bf.bibliography_export(format='XML').strip()
        expected_source = open(setup.MEDLINE_TEST_XML, 'r').read().strip()
        # just in case we need to debug that again        
        #l1 = bib_source.splitlines(1)
        #l2 = expected_source.splitlines(1)
        #from difflib import Differ
        #from pprint import pprint
        #d = Differ()
        #pprint(list(d.compare(l1,l2)))
        self.assertEqual(xml_source, expected_source)

    # end of the individual tests


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestRenderXML))
    return suite

if __name__ == '__main__':
    framework()



##########################################################################
#                                                                        #
#           copyright (c) 2003 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Preprint reference main class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import StringField, SelectionWidget
    from Products.LinguaPlone.public import DisplayList
    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import StringField, SelectionWidget
    from Products.Archetypes.public import DisplayList
    from Products.Archetypes.public import registerType
    
    
from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.base import BaseEntry
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema


SourceSchema = Schema((
    StringField('preprint_server',
        searchable=1,
        required=0,
        vocabulary="PreprintServers",
        widget=SelectionWidget(label="Preprint server",
                label_msgid="label_preprint_server",
                description="If the preprint is available from one of the following preprint servers, you can indicate that here. Contact the site's admin if you want a server to be added to the list.",
                description_msgid="help_preprint_server",
                i18n_domain="cmfbibliographyat",),
                ),
    ))

PreprintSchema = HeaderSchema.copy() + AuthorSchema.copy() + CoreSchema.copy() +  \
                 SourceSchema.copy() + TrailingSchema.copy()
                 
# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
PreprintSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }

class PreprintReference(BaseEntry):
    """ content type to make reference to a preprint
    """
    security = ClassSecurityInfo()
    archetype_name = "Preprint Reference"
    source_fields = ('preprint_server', )

    schema = PreprintSchema
    
    security.declareProtected(View, 'PreprintServers')
    def PreprintServers(self):
        """ return a display list of the preprint servers as
            defined in the tool properties.
        """
        bib_tool = getToolByName(self, 'portal_bibliography')
        servers = bib_tool.getProperty('preprint_servers', [])
        values = []
        for server in servers:
            if ':' in server:
                text = server.split(':')[0]
            else:
                text = server
            values.append([server, text])
        return DisplayList(tuple(values))
    
    security.declareProtected(View, 'Source')
    def Source(self):
        """ the default preprint server source format
        """
        try:
            return self.PreprintSource()
        except AttributeError:
            source = "Preprint"
            preprint_server = self.getPreprint_server()
            if preprint_server:
                if ':' in preprint_server:
                    text, link = preprint_server.split(':', 1)
                    if text != 'None':
                        source += ' at <a href="%s">%s</a>' % (link, text)
                else:
                    source += " at %s" % preprint_server
            return source + "."

registerType(PreprintReference, PROJECTNAME)



from Products.CMFBibliographyAT.content.folder \
    import BibliographyFolder, LargeBibliographyFolder
from Products.CMFBibliographyAT.content.article \
    import ArticleReference, ArticleSchema
from Products.CMFBibliographyAT.content.book \
    import BookReference, BookSchema
from Products.CMFBibliographyAT.content.booklet \
    import BookletReference, BookletSchema
from Products.CMFBibliographyAT.content.inbook \
    import InbookReference, InbookSchema
from Products.CMFBibliographyAT.content.incollection \
    import IncollectionReference
from Products.CMFBibliographyAT.content.inproceedings \
    import InproceedingsReference
from Products.CMFBibliographyAT.content.manual \
    import ManualReference, ManualSchema
from Products.CMFBibliographyAT.content.mastersthesis \
    import MastersthesisReference, MastersthesisSchema
from Products.CMFBibliographyAT.content.misc \
    import MiscReference, MiscSchema
from Products.CMFBibliographyAT.content.phdthesis \
    import PhdthesisReference, PhdthesisSchema
from Products.CMFBibliographyAT.content.preprint \
    import PreprintReference, PreprintSchema
from Products.CMFBibliographyAT.content.proceedings \
    import ProceedingsReference, ProceedingsSchema
from Products.CMFBibliographyAT.content.techreport \
    import TechreportReference, TechreportSchema
from Products.CMFBibliographyAT.content.unpublished \
    import UnpublishedReference
from Products.CMFBibliographyAT.content.webpublished \
    import WebpublishedReference

from Products.CMFBibliographyAT.tool.parsers.base import BibliographyParser
from Products.CMFBibliographyAT.tool.parsers.bibtex import BibtexParser
from Products.CMFBibliographyAT.tool.parsers.endnote import EndNoteParser
from Products.CMFBibliographyAT.tool.parsers.ibss import IBSSParser
from Products.CMFBibliographyAT.tool.parsers.isbn import ISBNParser
from Products.CMFBibliographyAT.tool.parsers.medline import MedlineParser
from Products.CMFBibliographyAT.tool.parsers.pyblbibtex \
    import PyBlBibtexParser
from Products.CMFBibliographyAT.tool.parsers.ris import RISParser
from Products.CMFBibliographyAT.tool.parsers.xml import XMLParser

from Products.CMFBibliographyAT.tool.renderers.base \
    import BibliographyRenderer
from Products.CMFBibliographyAT.tool.renderers.bibtex import BibtexRenderer
from Products.CMFBibliographyAT.tool.renderers.endnote import EndRenderer
from Products.CMFBibliographyAT.tool.renderers.pdf import PDFRenderer
from Products.CMFBibliographyAT.tool.renderers.ris import RISRenderer
from Products.CMFBibliographyAT.tool.renderers.xml import XMLRenderer



## Controller Python Script "bibliography_entry_addOwner"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=with_disabled=False
##title=
##

from Products.CMFCore.permissions import ModifyPortalContent

bib_tool = context.portal_bibliography
plone_utils = context.plone_utils
mtool = context.portal_membership

if context.portal_type in bib_tool.getReferenceTypes() and mtool.checkPermission(ModifyPortalContent, context):

    bibfolder = context.getBibFolder()
    	
    # id cooking
    if (bibfolder.getCookIdsOnBibRefCreation() and plone_utils.isIDAutoGenerated(context.getId())) or bibfolder.getCookIdsAfterBibRefEdit():
	new_id = bib_tool.cookReferenceId(ref=context, idcooker_id=bibfolder.getReferenceIdCookingMethod(), with_disabled=with_disabled)
	if new_id != 'nobody1000':
	    while bibfolder.hasObject(new_id) and new_id != context.getId():
		new_id = bibfolder.nextId(new_id)
	    bib_tool.transaction_savepoint(optimistic=True)
	    
	    # as of plone-2.1.3 this will cause an uncatalog warning in the logs if member reference support
	    # is enabled in the bib tool. Should be mended within Archetypes' reference engine...
            context.setId(value=new_id)	    




#######################################################
#                                                     #
#    Copyright (C), 2004, Raphael Ritz                #
#    <r.ritz@biologie.hu-berlin.de>                   #
#                                                     #
#    Humboldt University Berlin                       #
#                                                     #
# Copyright (C), 2005, Logilab S.A. (Paris, FRANCE)   #
# http://www.logilab.fr/ -- mailto:contact@logilab.fr #
#                                                     #
#######################################################

__revision__ = '$Id:  $'


import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup
from ZODB.PersistentList import PersistentList
from Globals import PersistentMapping

from Products.CMFBibliographyAT.tests import dummy

#ZopeTestCase.installProduct('DeadlockDebugger')
#import Products.DeadlockDebugger

class TestBibliographyFolder(PloneTestCase.PloneTestCase):
    '''Test the BibliographyFolder Import and duplication handling'''
    
    def afterSetUp(self):
        self._refreshSkinData()

        
    def getEmptyBibFolder(self, id='bibfolder'):

	uf = self.folder    
	if not hasattr(uf, id):
	    uf.invokeFactory(type_name = "BibliographyFolder", id=id )
	bf = getattr(uf, id)
        return bf
    	        
    def getPopulatedBibFolder(self, id='bibfolder'):
	bf = self.getEmptyBibFolder(id=id)    
	bib_source = open(setup.BIBTEX_TEST_BIB, 'r').read()
        bf.processImport(bib_source, 'bibtex_test.bib')
	return bf
	
    def getTwoPopulatedBibFolders(self, id1='bibfolder1', id2='bibfolder2'):
	bf1 = self.getPopulatedBibFolder(id=id1)
	bf2 = self.getPopulatedBibFolder(id=id2)
	return bf1, bf2
	
    def testSiteWideEnableDuplicatesManager(self):
    
	bf = self.getEmptyBibFolder(id='dup_tool_test')
	self.failUnless(bf.getEnableDuplicatesManager())

    def testDuplicatesFolder(self):
	bf = self.getEmptyBibFolder(id='dup_bf')
	bf.getDuplicatesFolder()
	self.failUnless('duplicates' in bf.contentIds())
	self.failUnless(bf.duplicates.portal_type == 'DuplicatesBibliographyFolder')
	self.failUnless(bf.duplicates.getReferenceIdCookingMethod() == 'uid')
	self.failUnless(bf.duplicates.getCookIdsAfterBibRefEdit())

	try: 
	    bf.duplicates.invokeFactory(type_name='ArticleReference', id='test1')
	except:
	    pass
	    
	# bib ref item creation should not be possible!!!
	self.failUnless('test1' not in bf.duplicates.contentIds())
	
    def testDuplicatesIdCooking(self):	
	bf = self.getEmptyBibFolder(id='dup_bf')
	df = bf.getDuplicatesFolder()
	bib_tool = bf.portal_bibliography
	
	tt = bf.portal_types
	fti = tt[bf.duplicates.portal_type]
	fti.allowed_content_types = tuple(bib_tool.getReferenceTypes())

	bf.duplicates.invokeFactory(type_name='ArticleReference', id='test1')
	bf.duplicates.invokeFactory(type_name='ArticleReference', id='test2')

	fti.allowed_content_types = ()

	test1 = bf.duplicates.test1
	test1.edit(title='Test1', publication_year='2001', journal='journal1')
	self.failUnless(test1.Title() == 'Test1')
	self.failUnless(test1.getPublication_year() == '2001')
	self.failUnless(test1.getJournal() == 'journal1')
	test1.bibliography_entry_cookId()
	
	test2 = bf.duplicates.test2
	test2.edit(title='Test2', publication_year='2002', journal='journal2')
	self.failUnless(test2.Title() == 'Test2')
	self.failUnless(test2.getPublication_year() == '2002')
	self.failUnless(test2.getJournal() == 'journal2')
	test2.bibliography_entry_cookId()

	for dup_id in bf.duplicates.contentIds():
	    self.failUnless(dup_id == bf.duplicates[dup_id].UID())

    def testDuplicatesFolderIdCooking(self):
	bf = self.getEmptyBibFolder(id='dup_bf')
	df = bf.getDuplicatesFolder()
	tt = bf.portal_types
	fti = tt[bf.duplicates.portal_type]
		
    def testObjectsMovedToAnotherBibFolderBecomeLocalDuplicatesOfTargetBibFolder(self):
    
	bf1, bf2 = self.getTwoPopulatedBibFolders(id1='move_bf1', id2='move_bf2')
	objs = bf1.manage_cutObjects(['Lutz2001'])
	bf2.manage_pasteObjects(objs)
	self.failUnless(len(bf1.contentIds()) == 2)
	self.failUnless(len(bf2.contentIds()) == 4)
	self.failUnless('duplicates' in bf2.contentIds())
	
	objs = bf1.manage_cutObjects(['McKay2003'])
	bf2.manage_pasteObjects(objs)
	self.failUnless(len(bf1.contentIds()) == 1)
	self.failUnless(len(bf2.contentIds()) == 4)
	self.failUnless('duplicates' in bf2.contentIds())
	
	for dup_id in bf2.duplicates.contentIds():
	    self.failUnless(dup_id == bf2.duplicates[dup_id].UID())
	
    def testObjectsCopiedToAnotherBibFolderBecomeLocalDuplicatesOfTargetBibFolder(self):
    
	bf1, bf2 = self.getTwoPopulatedBibFolders(id1='copy_bf1', id2='copy_bf2')
	objs = bf1.manage_copyObjects(['Lutz2001'])
	bf2.manage_pasteObjects(objs)
	self.failUnless(len(bf1.contentIds()) == 3)
	self.failUnless(len(bf2.contentIds()) == 4)
	self.failUnless('duplicates' in bf2.contentIds())
	
	objs = bf1.manage_copyObjects(['McKay2003'])
	bf2.manage_pasteObjects(objs)
	self.failUnless(len(bf1.contentIds()) == 3)
	self.failUnless(len(bf2.contentIds()) == 4)
	self.failUnless('duplicates' in bf2.contentIds())
	
	for dup_id in bf2.duplicates.contentIds():
	    self.failUnless(dup_id == bf2.duplicates[dup_id].UID())
        
    def testObjectsWithAssocPdfFilesMovedToAnotherBibFolderBecomeLocalDuplicatesOfTargetBibFolder(self):
    
	bf1, bf2 = self.getTwoPopulatedBibFolders(id1='move_bf1_pdf', id2='move_bf2_pdf')
	bf1.setSynchronizePdfFileAttributes(True)
	bf2.setSynchronizePdfFileAttributes(True)
	bf1.setCookIdsAfterBibRefEdit(True)
	bf2.setCookIdsAfterBibRefEdit(True)
	bib_tool = self.portal.portal_bibliography
	
	# upload some PDF files...
	pdffile_source = [
            dummy.File(filename='cmfbat-pdffile1.pdf', data=open(setup.CMFBAT_TEST_PDF_1, 'r').read(),),
	    dummy.File(filename='cmfbat-pdffile2.pdf', data=open(setup.CMFBAT_TEST_PDF_2, 'r').read(),),
	    dummy.File(filename='cmfbat-pdffile3.pdf', data=open(setup.CMFBAT_TEST_PDF_3, 'r').read(),),
	]
	idx1 = idx2 = 0 
	for item1, item2  in zip(bf1.contentValues(), bf2.contentValues()):
	    if item1.portal_type in bib_tool.getReferenceTypes():
	        item1.setUploaded_pdfFile(pdffile_source[idx1])
		idx1 += 1
	    if item2.portal_type in bib_tool.getReferenceTypes():
	        item2.setUploaded_pdfFile(pdffile_source[idx2])
		idx2 += 1
				
	# now do the cut+paste stuff again, but also check for the occurrence of the PDF file													    
	objs = bf1.manage_cutObjects(['Lutz2001'])
	bf2.manage_pasteObjects(objs)
	# expecting McKay2003, Lattier2001 and "pdfs" in bf1
	self.failUnless(len(bf1.contentIds()) == 3)
	self.failUnless('pdfs' in bf1.contentIds())
	# expecting McKay2003.pdf, Lattier2001.pdf in bf1.pdfs
	self.failUnless(len(bf1.pdfs.contentIds()) == 2)
	# expecting Lutz2001, McKay2003, Lattier2001, "pdfs" and "duplicates" in bf2
	self.failUnless(len(bf2.contentIds()) == 5)
	self.failUnless('pdfs' in bf2.contentIds())
	self.failUnless('duplicates' in bf2.contentIds())
	# expecting duplicate-Lutz2001 in bf2.duplicates
	self.failUnless(len(bf2.duplicates.contentIds()) == 1)
	# expecting Lutz2001.pdf, McKay2003.pdf, Lattier2001.pdf and <UID>.pdf for duplicate-Lutz2001 in bf2.pdfs
	self.failUnless(len(bf2.pdfs.contentIds()) == 4)
	
	objs = bf1.manage_cutObjects(['McKay2003'])
	bf2.manage_pasteObjects(objs)
	# expecting things dito
	self.failUnless(len(bf1.contentIds()) == 2)
	self.failUnless(len(bf2.contentIds()) == 5)
	self.failUnless(len(bf1.pdfs.contentIds()) == 1)
	self.failUnless(len(bf2.pdfs.contentIds()) == 5)
	self.failUnless(len(bf2.duplicates.contentIds()) == 2)
	
	for dup_id in bf2.duplicates.contentIds():
	    self.failUnless(dup_id == bf2.duplicates[dup_id].UID())
	    # uid cooker for duplicates' PDF files (so they do not interfere with other bibref items' ID cookers...)
	    self.failUnless('%s.pdf' % dup_id in bf2.pdfs.contentIds())
	
    def testObjectsWithAssocPdfFilesCopiedToAnotherBibFolderBecomeLocalDuplicatesOfTargetBibFolder(self):
    
	bf1, bf2 = self.getTwoPopulatedBibFolders(id1='copy_bf1_pdf', id2='copy_bf2_pdf')
	bf1.setSynchronizePdfFileAttributes(True)
	bf2.setSynchronizePdfFileAttributes(True)
	bf1.setCookIdsAfterBibRefEdit(True)
	bf2.setCookIdsAfterBibRefEdit(True)
	bib_tool = self.portal.portal_bibliography
	
	# upload some PDF files...
	pdffile_source = [
            dummy.File(filename='cmfbat-pdffile1.pdf', data=open(setup.CMFBAT_TEST_PDF_1, 'r').read(),),
	    dummy.File(filename='cmfbat-pdffile2.pdf', data=open(setup.CMFBAT_TEST_PDF_2, 'r').read(),),
	    dummy.File(filename='cmfbat-pdffile3.pdf', data=open(setup.CMFBAT_TEST_PDF_3, 'r').read(),),
	]
	idx1 = idx2 = 0 
	for item1, item2  in zip(bf1.contentValues(), bf2.contentValues()):
	    if item1.portal_type in bib_tool.getReferenceTypes():
	        item1.setUploaded_pdfFile(pdffile_source[idx1])
		idx1 += 1
	    if item2.portal_type in bib_tool.getReferenceTypes():
	        item2.setUploaded_pdfFile(pdffile_source[idx2])
		idx2 += 1
				
	# now do the cut+paste stuff again, but also check for the occurrence of the PDF file													    
	objs = bf1.manage_copyObjects(['Lutz2001'])
	bf2.manage_pasteObjects(objs)
	# expecting Lutz2001, McKay2003, Lattier2001 and "pdfs" in bf1
	self.failUnless(len(bf1.contentIds()) == 4)
	self.failUnless('pdfs' in bf1.contentIds())
	# expecting Lutz2001.pdf, McKay2003.pdf, Lattier2001.pdf in bf1.pdfs
	self.failUnless(len(bf1.pdfs.contentIds()) == 3)
	# expecting Lutz2001, McKay2003, Lattier2001, "pdfs" and "duplicates" in bf2
	self.failUnless(len(bf2.contentIds()) == 5)
	self.failUnless('pdfs' in bf2.contentIds())
	self.failUnless('duplicates' in bf2.contentIds())
	# expecting duplicate-Lutz2001 in bf2.duplicates
	self.failUnless(len(bf2.duplicates.contentIds()) == 1)
	# expecting Lutz2001.pdf, McKay2003.pdf, Lattier2001.pdf and <UID>.pdf for duplicate-Lutz2001 in bf2.pdfs
	self.failUnless(len(bf2.pdfs.contentIds()) == 4)
	
	objs = bf1.manage_copyObjects(['McKay2003'])
	bf2.manage_pasteObjects(objs)
	# expecting things dito
	self.failUnless(len(bf1.contentIds()) == 4)
	self.failUnless(len(bf2.contentIds()) == 5)
	self.failUnless(len(bf1.pdfs.contentIds()) == 3)
	self.failUnless(len(bf2.pdfs.contentIds()) == 5)
	self.failUnless(len(bf2.duplicates.contentIds()) == 2)
	
	for dup_id in bf2.duplicates.contentIds():
	    self.failUnless(dup_id == bf2.duplicates[dup_id].UID())
	    # uid cooker for duplicates' PDF files (so they do not interfere with other bibref items' ID cookers...)
	    self.failUnless('%s.pdf' % dup_id in bf2.pdfs.contentIds())
	
    def testObjectsWithAssocPdfFilesManuallyDeletedFromDuplicatesFolder(self):
    
	bf = self.getPopulatedBibFolder(id='del_bf_pdf')
	bf.setSynchronizePdfFileAttributes(True)
	bf.setCookIdsAfterBibRefEdit(True)
	bib_tool = self.portal.portal_bibliography
	
	# upload some PDF files...
	pdffile_source = [
            dummy.File(filename='cmfbat-pdffile1.pdf', data=open(setup.CMFBAT_TEST_PDF_1, 'r').read(),),
	    dummy.File(filename='cmfbat-pdffile2.pdf', data=open(setup.CMFBAT_TEST_PDF_2, 'r').read(),),
	    dummy.File(filename='cmfbat-pdffile3.pdf', data=open(setup.CMFBAT_TEST_PDF_3, 'r').read(),),
	]
	idx = 0 
	for item in bf.contentValues():
	    if item.portal_type in bib_tool.getReferenceTypes():
	        item.setUploaded_pdfFile(pdffile_source[idx])
		idx += 1
				
	# now do some copy+paste and then delete
	objs = bf.manage_copyObjects(['Lutz2001'])
	bf.manage_pasteObjects(objs)
	# expecting Lutz2001, McKay2003, Lattier2001, "pdfs" and "duplicates" in bf
	self.failUnless(len(bf.contentIds()) == 5)
	self.failUnless('pdfs' in bf.contentIds())
	self.failUnless('duplicates' in bf.contentIds())
	# expecting Lutz2001.pdf, McKay2003.pdf, Lattier2001.pdf and <UID of duplicate-Lutz2001.pdf> in bf.pdfs
	self.failUnless(len(bf.pdfs.contentIds()) == 4)
	# expecting duplicate-Lutz2001 in bf.duplicates
	self.failUnless(len(bf.duplicates.contentIds()) == 1)
	
	dup_bref_item = bf.duplicates.contentValues()[0]
	self.failUnless('%s.pdf' % dup_bref_item.UID() in bf.pdfs.contentIds())
	
	bf.duplicates.manage_delObjects([dup_bref_item.UID()])
	self.failUnless(len(bf.contentIds()) == 5)
	self.failUnless(len(bf.pdfs.contentIds()) == 3)
	self.failUnless(len(bf.duplicates.contentIds()) == 0)
	pdf_ids = bf.pdfs.contentIds()
	pdf_ids.sort()
	self.failUnless(tuple(pdf_ids) == ('Lattier2001.pdf', 'Lutz2001.pdf', 'McKay2003.pdf'))
	
	
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBibliographyFolder))
    return suite

if __name__ == '__main__':
    framework()



############################################################################
#                                                                          #
#             copyright (c) 2003 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""BIDSParser class"""

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# Bibliography stuff
from Products.CMFBibliographyAT.tool.parsers.base \
     import IBibliographyParser, BibliographyParser
from Products.CMFBibliographyAT.tool.parsers.base \
     import EntryParseError

# Python standard library imports
import re
from cStringIO import StringIO
from types import StringType


# mapping from IBSS keys to CMBBib keys
key_mapping = {  'TI'   : 'title'
               , 'AB'   : 'abstract'
              }

# keys that are (potentially) returned from _parseXX methods
parse_key_list = [  'publication_year'
                  , 'journal'
                  , 'publication_month'
                  , 'volume'
                  , 'number'
                  , 'pages'
                  , 'editor'
                  , 'chapter'
                  , 'authors'
                  , 'abstract'
                  , 'publisher'
                  , 'address'
                  , 'booktitle']


year_re = re.compile(', (\d\d\d\d)')
month_year_re = re.compile(', ([a-zA-Z\-]+) (\d\d\d\d),')
p_re = re.compile('(\d+)p\.')
p_re2 = re.compile('p\.(\d+)')
pp_re = re.compile('pp\.(\d+-\d+)')
vol_no_re = re.compile('Vol\.(\d+)\((\d+)\)')
vol_re = re.compile('Vol\.(\d+),')
no_re = re.compile('No.(\d+),')


class ParseException(Exception):
    pass


class IBSSParser(BibliographyParser):
    """
    A specific parser to process input in BIDS (IBSS) format.
    """

    __implements__ = (IBibliographyParser ,)

    meta_type = "IBSS Parser"

    format = {'name':'IBSS',
              'extension':'ibss'}

    def __init__(self,
                 id = 'bids',
                 title = "IBSS parser",
                 delimiter = '\n\n',
                 #pattern = r'(^.{0,4}- )',
                 flag = re.M):
        """
        initializes including the regular expression patterns
        """
        self.id = id
        self.title = title
        self.setDelimiter(delimiter)
        #self.setPattern(pattern, flag)


    # Here we need to provide 'checkFormat' and 'parseEntry'

    def checkFormat(self, source):
        """
        is this my format?
        """
        #raise Exception(type(source))
        if source.find('      AU: ') > -1:
            return 1
        else:
            return 0


    def parseEntry(self, entry):
        """
        parses a single entry
        
        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """
        if isinstance(entry, StringType):
            entry = StringIO(entry)
        key = ''
        val = []
        record = {}
        for line in entry.readlines():
            # XXX This should be made to work when there is not indentation!
            if line[8] == ':':
                record[key] = '\n'.join(val)
                key = line[6:8]
                val = []
                val.append(line[10:].strip())
            else:
                val.append(line.strip())

        result = {}

        result['authors'] = self._parseAU(record['AU'])

        for key in key_mapping.keys():
            if record.has_key(key):
                result[key_mapping[key]] = record[key]

        try:
            if record.has_key('JN'):
                result['reference_type'] = 'ArticleReference'
                r = self._parseJN(record['JN'])
            elif record.has_key('BN'):
                if record['DT'] == 'Chapter':
                    result['reference_type'] = 'InbookReference'
                    #r = self._parseBNCH(record['BN'])
                else:
                    result['reference_type'] = 'BookReference'
                r = self._parseBN(record['BN'])

            for key in parse_key_list:
                if r.has_key(key):
                    result[key] = r[key]
        except ParseException:
            # can be thrown by the _parseXX methods too
            # Must catch TypeError to stop it propagating up to masquerade as a checkFormat error <grr />
            entry.seek(0)
            result = EntryParseError(entry.read().replace('\n', '\\n'))
        except TypeError:
            entry.seek(0)
            result = EntryParseError('TYPEERROR: %s' % entry.read().replace('\n', '\\n'))

        return result

    def _parseAU(self, s):
        # e.g. 'Diermeier_D, Eraslan_H, Merlo_A'
        d = []
        s = s.replace('\n', ' ')
        l = s.split(', ')
        for person in l:
            p = {}
            p['lastname'], inits = person.split('_')
            if len(inits) > 1:
                p['firstname'] = inits[0]
                p['middlename'] = inits[1:]
            else:
                p['firstname'] = inits
            d.append(p)
        return d

    def _parseJN(self, s):
        # e.g.
        #   'Journal of politics, May 2002, Vol.64, No.2, pp.491-509'
        # or
        #   'National Institute economic review, Jan 2000, No.171, pp.94-105'
        # or
        #   'British journal of political science, Apr 1979, Vol.9(2), pp.129-156'
        d = {}
        s = s.replace('\n', ' ')
        p = pp_re.search(s)
        if p is None:
            p = p_re.search(s)
        if p is None:
            p = p_re2.search(s)
        if p is not None:
            d['pages'] = p.groups()[0]
        else:
            d['pages'] = ''
        p = vol_no_re.search(s)
        if p is not None:
            d['volume'] = p.groups()[0]
            d['number'] = p.groups()[1]
        else:
            p = vol_re.search(s)
            if p is not None:
                d['volume'] = p.groups()[0]
            else:
                d['volume'] = ''
            p = no_re.search(s)
            if p is not None:
                d['number'] = p.groups()[0]
            else:
                d['number'] = ''
        p = month_year_re.search(s)
        if p is not None:
            d['publication_month'] = p.groups()[0]
            d['publication_year'] = p.groups()[1]
            s = s[:p.span()[0]].strip()
        else:
            p = year_re.search(s)
            if p is not None:
                d['publication_year'] = p.groups()[-1]
                s = s[:p.span()[0]].strip()
        d['journal'] = s
        return d

    def _parseBN(self, s):
        # e.g.
        #   Economics and politics - the calculus of support, The 
        #   University of Michigan Press, Ann Arbor, M.I., 1991. pp.141-160
        # or
        #   Public support for market reforms in new democracies, Cambridge
        #   University Press, Cambridge, New York, 2001
        # or
        #   Inside local government: a case for radical reform, S. Browne, London, 1984. 414p.
        # or
        #   Party government in 48 democracies (1945-1998): composition, 
        #   duration, personnel, Kluwer Academic Publishers, Dordrecht, 
        #   Boston MA, 2000. xii, 580p.
        # I don't know how to deal with the latter.  Splitting on commas doesn't work :-(
        b = {}
        s = s.replace('\n', ' ')
        p = p_re.search(s)
        if p is None:
            p = pp_re.search(s)
        if p is not None:
            b['pages'] = p.groups()[0]
        else:
            b['pages'] = ''
        p = year_re.search(s) 
        # Strip off what we've already extracted while we're at the next step.
        if p is not None:
            b['publication_year'] = p.groups()[-1]
            s = s[:p.span()[0]].strip()
            #raise Exception(s)
        else:
            year = s[-4:]
            if year.isalnum():
                b['publication_year'] = year
                s = s[:-4].strip()
            else:
                b['publication_year'] = ''
        if s[-1] == ',':
            s = s[:-1]
        l = s.split(', ')
        # XXX Some assumptions are made now, and they may not always be right!
        if len(l) == 4:
            # Assumes (I think the more common case) s looks like the first 2 examples above.
            b['booktitle'] = l[0]
            b['publisher'] = l[1]
            b['address'] = ', '.join( (l[2], l[3]) )
        elif len(l) == 3:
            # Assumes s looks like the 3rd example above
            b['booktitle'] = l[0]
            b['publisher'] = l[1]
            b['address'] = l[2]
        elif len(l) > 4:
            b['address'] = l.pop()
            b['publisher'] = l.pop()
            b['booktitle'] = ', '.join(l)
        else:
            raise ParseException()
        return b

InitializeClass(IBSSParser)

   
def manage_addIBSSParser(self, REQUEST=None):
    """ """
    try:
        self._setObject('ibss', IBSSParser())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add already exists.',
            action='manage_main')
    return self.manage_main(self, REQUEST)



##########################################################################
#                                                                        #
#           copyright (c) 2003, 2005 ITB, Humboldt-University Berlin     #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""Article reference main class"""

from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View

from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import Schema
    from Products.LinguaPlone.public import StringField, StringWidget
#    from Products.LinguaPlone.public import registerType
else:
    from Products.Archetypes.public import Schema
    from Products.Archetypes.public import StringField, StringWidget
from Products.Archetypes.public import registerType

from Products.CMFBibliographyAT.marshall import BibtexMarshaller
from Products.CMFBibliographyAT.config import PROJECTNAME
from Products.CMFBibliographyAT.content.base import BaseEntry
from Products.CMFBibliographyAT.content.schemata \
    import HeaderSchema, AuthorSchema, CoreSchema, TrailingSchema
from Products.CMFBibliographyAT.content.fields \
    import journalField, volumeField, numberField, pagesField


SourceSchema = Schema((
    journalField,
    volumeField,
    numberField,
    pagesField,
    StringField('pmid',
	is_duplicates_criterion=True,
        widget=StringWidget(label="PubMed ID",
            label_msgid="label_pmid",
            description="The reference's number in the PubMed database.",
            description_msgid="help_pmid",
            i18n_domain="cmfbibliographyat",
	),
    ),
    StringField('DOI',
	is_duplicates_criterion=True,
        widget=StringWidget(label="DOI",
            label_msgid="label_doi",
            description="The reference's digital object identifier.",
            description_msgid="help_doi",
            i18n_domain="cmfbibliographyat",
	),
    ),
))

ArticleSchema = HeaderSchema.copy() + AuthorSchema.copy() + CoreSchema.copy() +  \
                SourceSchema.copy() + TrailingSchema.copy()
ArticleSchema.get('authors').required = 1
ArticleSchema.get('publication_year').required = 1
ArticleSchema.get('journal').required = 1

# the default AT 'description' field shall be invisible, it is kept in sync with the 'abstract' field
ArticleSchema.get('description').widget.visible = {'view': 'invisible', 'edit': 'invisible', }

class ArticleReference(BaseEntry):
    """ content type to make reference to a (scientific) article.
    """
    security = ClassSecurityInfo()
    archetype_name = "Article Reference"
    source_fields = ('journal', 'volume', 'number', 'pages', 'pmid',)

    schema = ArticleSchema
    
    security.declareProtected(View, 'PMID')
    def PMID(self):
        """ returns the pmid if set
        """
        value = self.getPmid()
        if value:
            return value
        else:
            return None
        
    security.declareProtected(View, 'getPubMedLink')
    def getPubMedLink(self, defaultURL=None):
        """ a link to PubMed
            if pmid is set or the default otherwise
        """
        if self.getPmid():
            url = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Retrieve"
            return url + "&db=PubMed&list_uids=%s" % self.getPmid()
        else: return defaultURL

    security.declareProtected(View, 'Source')
    def Source(self):
        """ the default article source format
        """
        try:
            return self.ArticleSource()
        except AttributeError:
            journal = self.getJournal()
            volume  = self.getVolume()
            number  = self.getNumber()
            pages   = self.getPages()
            source = ''
            if journal:
                source += journal
            if volume:
                source += ', %s' % volume
            if number:
                source += '(%s)' % number
            if pages:
                source += ':%s' % pages
            return source + '.'

registerType(ArticleReference, PROJECTNAME)



from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
if CMFBAT_USES_LINGUAPLONE:
    from Products.LinguaPlone.public import StringField, StringWidget
    from Products.LinguaPlone.public import BooleanField, BooleanWidget
else:
    from Products.Archetypes.public import StringField, StringWidget
    from Products.Archetypes.public import BooleanField, BooleanWidget


journalField = StringField('journal',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Journal",
                    label_msgid="label_journal",
                    description="A journal name (should be the full journal name).",
                    description_msgid="help_journal",
                    i18n_domain="cmfbibliographyat",),
                )

volumeField = StringField('volume',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Volume",
                    label_msgid="label_volume",
                    description="The volume of a journal, multivolume book work etc. In most cases it should suffice to use a simple number as the volume identifier.",
                    description_msgid="help_volume",
                    i18n_domain="cmfbibliographyat",),
                )

numberField = StringField('number',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Number",
                    label_msgid="label_number",
                    description="The number of a journal, proceedings, technical report etc. Issues of journals, proceedings etc. are usually identified by volume and number; however, issues of the same volume will often be numbered preceedingly which often makes the specification of a number optional. With technical reports, the issuing organization usually also gives it a number.",
                    description_msgid="help_number",
                    i18n_domain="cmfbibliographyat",),
                )

pagesField = StringField('pages',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Pages",
                    label_msgid="label_pages",
                    description="A page number or range of numbers such as '42-111'; you may also have several of these, separating them with commas: '7,41,73-97'.",
                    description_msgid="help_pages",
                    i18n_domain="cmfbibliographyat",),
                )

howpublishedField = StringField('howpublished',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="How published",
                    label_msgid="label_howpublished",
                    description="For publications without a publisher. e.g., an 'Institute Report'.",
                        description_msgid="help_howpublished",
                        i18n_domain="cmfbibliographyat",),
                )

publisherField = StringField('publisher',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Publisher",
                    label_msgid="label_publisher",
                    description="The publisher's name.",
                    description_msgid="help_publisher",
                    i18n_domain="cmfbibliographyat",),
                )

addressField = StringField('address',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Address",
                    label_msgid="label_address",
                    description="Publisher's address. For major publishing houses, just the city is given. For small publishers, you can help the reader by giving the complete address.",
                    description_msgid="help_address",
                    i18n_domain="cmfbibliographyat",),
                )

editionField = StringField('edition',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Edition",
                    label_msgid="label_edition",
                    description="The edition of a book - for example: 'II', '2' or 'second', depending on your preference. Numbers will be turned into ordinal numerals.",
                    description_msgid="help_edition",
                    i18n_domain="cmfbibliographyat",),
                )

booktitleField = StringField('booktitle',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Book Title",
                       label_msgid="label_booktitle",
                       description="Title of the book, collection or proceedings, that the cited resource is part of.",
                       description_msgid="help_booktitle",
                       i18n_domain="cmfbibliographyat",),
                )

editorField = StringField('editor',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Editor",
                    label_msgid="label_editor",
                    description="Name(s) of editor(s). Opposed to the 'author' field, the 'editor' field should contain the editor of the book, collection or proceeding that the reference appears in.",
                    description_msgid="help_editor",
                    i18n_domain="cmfbibliographyat",),
                )

seriesField = StringField('series',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Series",
                    label_msgid="label_series",
                    description="The name of a series or set of books. When citing an entire book, the 'title' field gives its title and this optional 'series' field gives the name of a series in which the book is published.",
                    description_msgid="help_series",
                    i18n_domain="cmfbibliographyat",),
                )

chapterField = StringField('chapter',
                searchable=0,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Chapter",
                    label_msgid="label_chapter",
                    description="A chapter number.",
                    description_msgid="help_chapter",
                    i18n_domain="cmfbibliographyat",),
                )

organizationField = StringField('organization',
                searchable=1,
                required=0,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Organization",
                    label_msgid="label_organization",
                    description="The organization sponsoring a conference, issuing a technical report etc.",
                    description_msgid="help_organization",
                    i18n_domain="cmfbibliographyat",),
                )

schoolField = StringField('school',
                required=0,
                searchable=1,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="School",
                    label_msgid="label_school",
                    description="The name of the school (college, university etc.) where a thesis was written.",
                    description_msgid="help_school",
                    i18n_domain="cmfbibliographyat",),
                )

institutionField = StringField('institution',
                required=0,
                searchable=1,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Institution",
                    label_msgid="label_institution",
                    description="The institution that published the work.",
                    description_msgid="help_institution",
                    i18n_domain="cmfbibliographyat",),
                )

typeField = StringField('publication_type',
                required=0,
                searchable=1,
                default='',
		is_duplicates_criterion=True,
                widget=StringWidget(label="Type",
                    label_msgid="label_publication_type",
                    description="The type of a technical report, PhD thesis etc. - for example, 'Research Note' or 'Doktorarbeit'.",
                    description_msgid="help_publication_type",
                    i18n_domain="cmfbibliographyat",),
                )

editor_flagField = BooleanField('editor_flag',
                default=0,
                required=0,
		is_duplicates_criterion=True,
                widget=BooleanWidget(label="Editor Flag",
                    label_msgid="label_editor_flag",
                    description="Check here if the author(s) specified above are actually the editor(s) of the book.",
                    description_msgid="help_editor_flag",
                    i18n_domain="cmfbibliographyat",)
                )



## Script (Python) "getPortalTypeNames"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
type_ids = context.portal_types.listTypeTitles().keys()
type_ids.sort()
return type_ids



from Products.bibTypes.config import *
from Products.bibTypes.base_fields import *
from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions

schema = BaseSchema +  Base_Schema + Schema((

    StringField('Journal',
                required=0,
                searchable=1,
                widget=StringWidget(description="The title of the journal.",
                                    label="Journal Title",),
                ),

    StringField('Volume',
                  required=0,
                  searchable=0,
                  widget=StringWidget(description="Enter the volume of the journal.",
                                       label="Volume"),
                  ),

    StringField('Issue',
                  required=0,
                  searchable=0,
                  widget=StringWidget(description="Enter the issue of the journal.",
                                       label="Issue"),
                  ),

    StringField('Pages',
                  required=0,
                  searchable=0,
                  widget=StringWidget(description="Enter the page numbers of the article.",
                                       label="Pages"),
                  ),

    StringField('Publisher',
                required=0,
                searchable=1,
                widget=StringWidget(description="The name of the publisher.",
                                    label="Publisher",),
                ),

    IntegerField('PublishYear',
                  required=0,
                  searchable=0,
                  validators=('isInt',),
                  widget=IntegerWidget(description="Enter the year of publication.",
                                       label="Publication Year"),
                  ),

    StringField('PublishMonth',
                vocabulary=MONTH,
                enforceVocabulary=1,
                required=0,
                searchable=0,
                widget=SelectionWidget(description="Select the month of publication.",
                                       label="Publication Month"),
                ),

    StringField('ISSN',
                required=0,
                searchable=0,
                widget=StringWidget(description="The ISSN of the journal.",
                                    label="ISSN"),
                ),
    )) + Trailing_Schema

actions = (
    {
    'id' : 'view',
    'name' : 'View',
    'action' : 'base_view',
    'permissions' : (CMFCorePermissions.View, )
    },
    ) + base_actions

class Article(BibSchema):
    """A bibliographic record of a journal article"""
    schema = schema
    actions = actions
    include_default_actions = 0
    global_allow = 0
    filter_content_types = 1
    archetype_name = "Journal article"    

registerType(Article)



# -*- encoding: utf-8 -*-
#######################################################
#                                                     #
#    Copyright (C), 2004, Raphael Ritz                #
#    <r.ritz@biologie.hu-berlin.de>                   #
#                                                     #
#    Humboldt University Berlin                       #
#                                                     #
# Copyright (C), 2005, Logilab S.A. (Paris, FRANCE)   #
# http://www.logilab.fr/ -- mailto:contact@logilab.fr #
#                                                     #
#######################################################

__revision__ = '$Id:  $'


import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup
from ZODB.PersistentList import PersistentList
from Globals import PersistentMapping

#ZopeTestCase.installProduct('DeadlockDebugger')
#import Products.DeadlockDebugger

class TestBibtexParsing(PloneTestCase.PloneTestCase):
    """
    """
    
    def afterSetUp(self):
        self._refreshSkinData()
        #self.bf = self.getEmptyBibFolder()
        #bib_source = open(setup.BIBTEX_TEST_BIB, 'r').read()
        #self.bf.processImport(bib_source, 'bibtex_test.bib')

    def testBibtexAuthorParsing(self):
        bibtool = self.portal.portal_bibliography
        p = bibtool.getParser('bibtex')
        source = open(setup.BIBTEX_TEST_MULTI_AUTHORS, 'r').read()
        source = p.preprocess(source)
        result = p.parseEntry(source)
        heckman =  {'middlename': 'J.',
                    'firstname' : 'James',
                    'lastname'  : 'Heckman'}
        carneiro = {'middlename': '',
                    'firstname' : 'Pedro',
                    'lastname'  : 'Carneiro'}
        self.failUnless( len( result['authors'] ) == 2 )
        author1 = result['authors'][0]
        self.failUnless(author1['middlename'] == carneiro['middlename'])
        self.failUnless(author1['firstname'] == carneiro['firstname'])
        self.failUnless(author1['lastname'] == carneiro['lastname'])
        author2 = result['authors'][1]
        self.failUnless(author2['middlename'] == heckman['middlename'])
        self.failUnless(author2['firstname'] == heckman['firstname'])
        self.failUnless(author2['lastname'] == heckman['lastname'])

    def testBibtexInbookReferenceParsing(self):
        bibtool = self.portal.portal_bibliography
        p = bibtool.getParser('bibtex')
        source = open(setup.BIBTEX_TEST_INBOOKREFERENCES, 'r').read()
	ref = {
	    'booktitle': 'In einem fiktiven Buch vor unserer Zeit',
	    'title': 'Die Tage der Ankunft',
	    'chapter': 'Die Tage der Ankunft',
	    'publication_url': 'http://www.sunweavers.net/',
	}

	source = p.preprocess(source)
    	result = p.parseEntry(source)
	
	for key in ref.keys():
	    self.failUnless( result.has_key(key) and (ref[key] == result[key]) )

    def testAnnoteParsing(self):
        bibtool = self.portal.portal_bibliography
        p = bibtool.getParser('bibtex')
        source = open(setup.BIBTEX_TEST_BIB, 'r').read()
        results = p.getEntries(source)
        self.failUnless(results[-1]['annote'] == 'I really like it.')

    def testBibtexTypeFieldParsing(self):
        bibtool = self.portal.portal_bibliography
        p = bibtool.getParser('bibtex')
        source = open(setup.BIBTEX_TEST_TYPEFIELD, 'r').read()
	ref = {
	    'publication_type': 'Doktorarbeit',
	    'title': 'Mein Herr Doktor',
	    'school': 'CAU Kiel',
	    'institution': 'Ökologie-Zentrum',
	}

	source = bibtool.checkEncoding(source)
	source = p.preprocess(source)
    	result = p.parseEntry(source)
	
	for key in ref.keys():
	    self.failUnless( result.has_key(key) and (ref[key] == result[key]) )

    def testBibtexTypeLastFieldTrailingKomma(self):
        bibtool = self.portal.portal_bibliography
        p = bibtool.getParser('bibtex')
        source = open(setup.BIBTEX_TEST_LASTFIELDKOMMA, 'r').read()

	source = bibtool.checkEncoding(source)
	source = p.preprocess(source)
	results = p.getEntries(source)

	# the last field in a bibtex entry always had a trailing ","
	self.failUnless( len(results) == 2  )
	self.failUnless( results[0]['institution'] == results[1]['institution']  )
	self.failUnless( results[0]['publication_type'] == results[1]['publication_type']  )
	self.failUnless( results[0]['publication_type'] == 'Doktorarbeit,,,' )

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestBibtexParsing))
    return suite

if __name__ == '__main__':
    framework()



"""Abbreviation Bibliography IdCooker class"""

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# CMF stuff
from Products.CMFCore.utils import getToolByName

# Bibliography stuff
from Products.CMFBibliographyAT.tool.idcookers.base \
     import IBibliographyIdCooker, BibliographyIdCooker


class AbbrevIdCooker(BibliographyIdCooker):
    """
    If a bibliographical reference item has one author, it takes the first three letters of the author's lastname
    plus publication year to cook the reference's ID. For more than one author, it uses the initials of all authors' lastnames plus publication
    year.
    """

    __implements__ = (IBibliographyIdCooker ,)

    meta_type = "Abbreviation Bibliography ID Cooker"

    def __init__(self,
                 id = 'etal',
                 title = "Abbreviation Bibliography ID Cooker"):
        """
        initializes id and title
        """
        self.id = id
        self.title = title

    def _cookIdCore(self, ref, **kwargs):
        """
        cooks a bibref id for one reference entry dict
        """

        # AUTHORS
	namepart='nobody'
        if self._refHasAuthorNames(ref):
            lastnames = []
            for each in ref['authors']:
                if each.get('lastname', None):
		    lastnames.append(each['lastname'])
            
	    if len(lastnames) > 1:
                namepart = '%s' % ''.join([ lastname[0] for lastname in lastnames ])
	    elif len(lastnames) == 1:
	        namepart = lastnames[0][:3]
	    else:
		pass	
	
	# PUBLICATION YEAR
	if ref.get('publication_year', None):
	    yearpart = str(ref['publication_year'])
	else:
	    yearpart = "1000"
	
	return  namepart + yearpart


InitializeClass(AbbrevIdCooker)


def manage_addAbbrevIdCooker(self, REQUEST=None):
    """ """
    try:
        self._setObject('etal', AbbrevIdCooker())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The id cooker you attempted to add already exists.',
            action='manage_main')
    return self.manage_main(self, REQUEST)



##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""
mixin class to support 'bibutils' based transformations

bibutils are wri
tten and maintained by Chris Putnam at Scripps
http://www.scripps.edu/~cdputnam/software/bibutils/bibutils.html
"""

import sys, os, os.path
from Globals import InitializeClass
from zLOG import LOG, ERROR
from AccessControl import ClassSecurityInfo
from Products.CMFCore.permissions import View

commands = {'bib2xml':'bib2xml',
            'copac2xml':'copac2xml',
            'end2xml':'end2xml',
            'isi2xml':'isi2xml',
            'med2xml':'med2xml',
            'ris2xml':'ris2xml',
            'xml2bib':'xml2bib',
            'xml2end':'xml2end',
            'xml2ris':'xml2ris',
            'copac2bib':'copac2xml | xml2bib ',
            'end2bib':'end2xml | xml2bib ',
            'isi2bib':'isi2xml | xml2bib ',
            'med2bib':'med2xml | xml2bib ',
            'ris2bib':'ris2xml | xml2bib ',
            'bib2end':'bib2xml | xml2end ',
            'bib2ris':'bib2xml | xml2ris ',            
            'copac2end':'copac2xml | xml2end ',
            'isi2end':'isi2xml | xml2end ',
            'med2end':'med2xml | xml2end ',
            'ris2end':'ris2xml | xml2end ',
            'end2ris':'end2xml | xml2ris ',            
            'copac2ris':'copac2xml | xml2ris ',
            'isi2ris':'isi2xml | xml2ris ',
            'med2ris':'med2xml | xml2ris ',
            }

def _getKey(source_format, target_format):
    return '2'.join([source_format, target_format])

class BibUtils:
    """mixin class to support bibutils based transformations"""
    
    security = ClassSecurityInfo()

    def __init__(self):
        pass

    security.declareProtected(View, 'isTransformable')
    def isTransformable(self, source_format, target_format):
        """ test if a transform from source_format to target_format would be feasible
        """
        key = _getKey(source_format, target_format)
	command = commands.get(key, None)
	if command is None:
	    return False
        commandlist = [ c.strip() for c in command.split('|') ]
	
	# open each command once
	transformable = True
	for c in commandlist:

	    pass
	    
	return transformable
    
    security.declareProtected(View, 'transform')
    def transform(self, data, source_format, target_format):
        """
        calling 'bibutils' to transform data from 'source_format'
        to 'target_format'
        """
        key = _getKey(source_format, target_format)
        command = commands.get(key, None)
        if command is None:
            raise ValueError, "No transformation from '%s' to '%s' found." \
                  % (source_format, target_format)

        (fi, fo, fe) = os.popen3(command, 't')
        fi.write(data)
        fi.close()
        result = fo.read()
        fo.close()
        error = fe.read()
        fe.close()
        if error:
            # command could be like 'ris2xml', or 'ris2xml | xml2bib'. It
            # seems unlikely, but we'll code for an arbitrary number of
            # pipes...
            command_list = command.split(' | ')
	    for each in command_list:
                if each in error and not result:
                    result = self._callFallback(data,
                                                source_format,
                                                target_format)
        return result

    def _callFallback(self, data, source_format, target_format):

        # call a service somewhere else
        # not implemented yet; for now just log and blow up
        key = _getKey(source_format, target_format)
        command = commands.get(key, None)
        msg = "'%s' not found. Make sure 'bibutils' is installed." \
              % command
        LOG('CMFBibliographyAT.tool.bibutils',
            ERROR,
            msg,
            )
        # XXX I don't think NameError is strictly the right exception,
        # but I can't think of a better one off the top of my head.
        # Perhaps OSError?
	raise NameError, msg

InitializeClass(BibUtils)




## Controller Python Script "bibliography_pdffile_cookId"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=with_disabled=False
##title=
##

from Products.CMFCore.permissions import ModifyPortalContent

bib_tool = context.portal_bibliography
types_tool = context.portal_types
plone_utils = context.plone_utils
mtool = context.portal_membership

if context.portal_type in bib_tool.getReferenceTypes() and mtool.checkPermission(ModifyPortalContent, context):

    bibfolder = context.aq_inner.aq_parent
    pdf_folder = bibfolder.getPdfFolder()
    pdf_file = context.getPdf_file()
			
    # id cooking for associated PDF files
    if pdf_file and bibfolder.getSynchronizePdfFileAttributes():
	
	if pdf_file.getId() != '%s.pdf' % context.getId():
	
	    # only if the IDs of BibRefItem and PDF file differ, we have to get into action
	    pdf_file_id = context.getId() + '.pdf'
	    if pdf_folder.hasObject(pdf_file_id):
		# arggghhh... this should not happen!!!
	        # let us try to cook an id that resembles the bibrefitem's ID (the least we can do)
		new_id = bib_tool.cookReferenceId(ref=context, idcooker_id=bibfolder.getReferenceIdCookingMethod(), with_disabled=with_disabled)
	        if new_id != 'nobody1000':
	    	    while pdf_folder.hasObject(new_id) and new_id != context.getPdf_file().getId():
			new_id = pdf_folder.nextId(new_id)
	        pdf_file_id = new_id + '.pdf'
	
	    bib_tool.transaction_savepoint(optimistic=True)
	
	    # we need to explicitly add 'PDF File' content type as allowed type to 'PDF Folder' content type
	    # even for pdf_file.setId (picky Plone!!!)
    	    pdf_folder.allowPdfFileCreation()

	    # set new id for PDF file
	    pdf_file.setId(value=pdf_file_id)

	    # restore PDF folder fti
    	    pdf_folder.disallowPdfFileCreation()
	



############################################################################
#                                                                          #
#             copyright (c) 2004 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""XMLParser (MODS) class"""

# Python stuff
import re, os

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog
from Products.CMFCore.utils import getToolByName

# Bibliography stuff
from Products.CMFBibliographyAT.tool.parsers.base \
     import IBibliographyParser, BibliographyParser

try:
    import _bibtex
    from Products.CMFBibliographyAT.tool.parsers.pyblbibtex \
         import PyBlBibtexParser as BaseParser
except ImportError:
    from Products.CMFBibliographyAT.tool.parsers.bibtex \
         import BibtexParser as BaseParser


class XMLParser(BaseParser):
    """
    A specific parser to process input in XML(MODS)-format. The XML intermediate format is conform to the
    Library of Congress's Metadata Object Description Schema (MODS). This is a very flexible standard that 
    should prove quite useful as the number of tools that directly interact with it increase. 
    """

    __implements__ = (IBibliographyParser,)

    meta_type = "XML (MODS) Parser"

    format = {'name':'XML (MODS)',
              'extension':'xml'}

    def __init__(self,
                 id = 'xml_mods',
                 title = "XML(MODS) parser"
                 ):
        """
        initializes including the regular expression patterns
        """
        BaseParser.__init__(self, id=id, title=title)   

    # Here we need to provide 'isAvailable', 'checkFormat' and 'preprocess'

    def isAvailable(self):
        """ test if transforming from XML to BibTex is possible...
        """
        bib_tool = getToolByName(self, 'portal_bibliography')
        return bib_tool.isTransformable('xml', 'bib')

    def checkFormat(self, source):
        """
        is this my format?
        """
        teststring = source[:200].lower()
        ai = teststring.find('www.loc.gov/mods')
        ei = teststring.find('modsCollection')
        if ai + ei > -2:
            return 1
        else:
            return 0

    def preprocess(self, source):
        """
        convert XML to BibTeX
        """
        tool = getToolByName(self, 'portal_bibliography')
        return tool.transform(source, 'xml', 'bib')
    
##         # open a pipe
##         (fi, fo, fe) = os.popen3('xml2bib ', 't')
##         # provide the input
##         fi.write(source)
##         fi.close()
##         # get the output
##         bibtex = fo.read()
##         fo.close()
##         # get the staus/error message
##         # (this isn't used but we don't want it in the output)
##         error = fe.read()
##         fe.close()
##         # done
##         return bibtex

    # all the rest we inherit from our parent BibTeX(!) parser

InitializeClass(XMLParser)

   
def manage_addXMLParser(self, REQUEST=None):
    """ """
    try:
        self._setObject('xml_mods', XMLParser())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add already exists.',
            action='manage_main')    
    return self.manage_main(self, REQUEST)



############################################################################
#                                                                          #
#             copyright (c) 2003 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""MedlineParser class"""

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog

# Bibliography stuff
from Products.CMFBibliographyAT.tool.parsers.base \
     import IBibliographyParser, BibliographyParser

import re


class MedlineParser(BibliographyParser):
    """
    A specific parser to process input in Medline-format.
    """

    __implements__ = (IBibliographyParser ,)

    meta_type = "Medline Parser"

    format = {'name':'Medline',
              'extension':'med'}

    def __init__(self,
                 id = 'medline',
                 title = "Medline parser",
                 delimiter = '\n\n',
                 pattern = r'(^.{0,4}- )',
                 flag = re.M):
        """
        initializes including the regular expression patterns
        """
        self.id = id
        self.title = title
        self.setDelimiter(delimiter)
        self.setPattern(pattern, flag)


    # Here we need to provide 'checkFormat' and 'parseEntry'

    def checkFormat(self, source):
        """
        is this my format?
        """
        # vanilla test for 'PMID- ' in the sub-string 'source[0, 100]'
        ## rr: can definitively be improved
        
        if source.find('PMID- ', 0, 1000) > -1:
            return 1
        else:
            return 0

    def parseEntry(self, entry):
        """
        parses a single entry
        
        returns a dictionary to be passed to
        BibliographyEntry's edit method
        """
        result = {}
        
        tokens = self.pattern.split(entry)

        checkAU = 0
        if 'FAU - ' not in tokens:
            checkAU = 1
            
        nested = []
        for i in range(1,len(tokens),2):
            nested.append([tokens[i],tokens[i+1]])

        # some defaults
        result['note'] = 'automatic medline import'

        for key, value in nested:
            if key == 'PT  - ' and (value.find('Journal Article')> -1
                                    or value.find('JOURNAL ARTICLE')> -1):
                result['reference_type'] = 'ArticleReference'
            elif key == 'TI  - ':
                title = value.replace('\n', ' ').replace('      ', '').strip()
                result['title'] = title
            elif key == 'AB  - ':
                tmp = value.replace('\n', ' ').replace('  ', '')
                result['abstract'] = tmp.replace('  ', '').replace('  ', '')
            elif key == 'PMID- ': result['pmid'] = str(value).strip()
            elif key == 'TA  - ': result['journal'] = str(value).strip()
            elif key == 'VI  - ': result['volume'] = str(value).strip()
            elif key == 'IP  - ': result['number'] = str(value).strip()
            elif key == 'PG  - ': result['pages'] = str(value).strip()
            elif key == 'DP  - ':
                result['publication_year'] = value[:4]
                pmonth = value[5:].replace('\n','').replace('\r','')
                result['publication_month'] = pmonth
            elif key == 'FAU - ':
                raw = value.replace('\n', '').split(', ')
                lname = raw[0]
                fnames = raw[1].split(' ',1)
                fname = fnames[0]
                if len(fnames)> 1:
                    minit = fnames[1]
                else:
                    minit = ''
                adict = {'firstname': fname
                         ,'middlename': minit
                         ,'lastname': lname
                         }
                result.setdefault('authors',[]).append(adict)

            elif checkAU and key == 'AU  - ':
                raw = value.replace('\n', '').split()
                lname = raw[0]
                fnames = raw[1]
                fname = fnames[0]
                if len(fnames)> 1:
                    minit = fnames[1:]
                else:
                    minit = ''
                adict = {'firstname': fname
                         ,'middlename': minit
                         ,'lastname': lname
                         }
                result.setdefault('authors',[]).append(adict)
    
        return result

InitializeClass(MedlineParser)

   
def manage_addMedlineParser(self, REQUEST=None):
    """ """
    try:
        self._setObject('medline', MedlineParser())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The parser you attempted to add already exists.',
            action='manage_main')
        
    
    return self.manage_main(self, REQUEST)



from Products.bibTypes.config import *
from Products.bibTypes.base_fields import *
from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions

schema = BaseSchema +  Base_Schema + Schema((

    StringField('Series',
                required=0,
                searchable=1,
                widget=StringWidget(description="The title of the series.",
                                    label="Series",),
                ),    

    StringField('Publisher',
                required=0,
                searchable=1,
                widget=StringWidget(description="The name of the publisher.",
                                    label="Publisher",),
                ),

    StringField('Institution',
                required=0,
                searchable=1,
                widget=StringWidget(description="The name of the institution.",
                                    label="Institution",),
                ),

    IntegerField('PublishYear',
                  required=0,
                  searchable=0,
                  validators=('isInt',),
                  widget=IntegerWidget(description="Enter the year of publication.",
                                       label="Publication date"),
                  ),

    StringField('ReportRef',
                required=0,
                searchable=0,
                widget=StringWidget(description="The reference number of the report.",
                                    label="Reference Number",),
                ),    

    )) + Trailing_Schema

actions = (
    {
    'id' : 'view',
    'name' : 'View',
    'action' : 'base_view',
    'permissions' : (CMFCorePermissions.View, )
    },
    ) + base_actions

class TechnicalReport(BibSchema):
    """A bibliographic record of a technical report"""
    schema = schema
    actions = actions
    include_default_actions = 0
    global_allow = 0
    filter_content_types = 1
    archetype_name = "Technical Report"

registerType(TechnicalReport)



## Script (Python) "getISBNLink"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=isbn=None
##title=
##
if not isbn: return ("", "")
isbn = isbn.replace('-', '').strip()
return ("Link to Amazon", "http://www.amazon.com/exec/obidos/tg/detail/-/%s" % isbn)



"""BibliographyIdCooker main class"""

import re

# Zope stuff
from Interface import Interface
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
import Products

# PLONE things
from Products.CMFCore.utils import getToolByName
from Products.CMFBibliographyAT.utils import _decode, _encode


class IBibliographyIdCooker(Interface):
    """
    Interface for the bibliography ID cookers
    in the bibliography tool.
    """
    def getId():
        """
        returns the id of the reference id cooker
        """

    def Title():
        """
        returns the title of the reference id cooker
        """

    def Description():
        """ 
        a short text that explains the basic functionality of the id cooker
        """

    def isEnabled():
        """
        is the renderer enabled by the portal manager (default: yes)?
        """
          
    def getCookedBibRefId():
        """
        returns a cooked id for a bibliography reference dictionary or object
        """

class BibliographyIdCooker(SimpleItem, PropertyManager):
    """
    Base class for the id cookers of the bibliography tool.
    """
    __implements__ = (IBibliographyIdCooker ,)

    meta_type = 'Bibliography IdCooker'
    idcooker_enabled = True
    
    manage_options = (  
        PropertyManager.manage_options + 
        SimpleItem.manage_options
    )            
    _properties = PropertyManager._properties + (
        {'id': 'idcooker_enabled',
         'type': 'boolean',
         'mode': 'w',
         },
    )
    
    security = ClassSecurityInfo()

    def __init__(self, id, title=''):
        """
        minimal initialization
        """
        self.id = id
        self.title = title

    def getId(self):
        """ returns the id of the id cooker """
        return self.id

    def Title(self):
        """ returns the title of the id cooker """
        domain='cmfbibliographyat'
        msgid='title_idcooker_%s' % self.getId().lower()
        return self.translate(domain=domain, msgid=msgid, default='%s' % self.title)

    def Description(self):
        """ 
        a short text that explains the basic functionality of the id cooker
        """
        domain='cmfbibliographyat'
        msgid='help_idcooker_%s' % self.getId()
        return self.translate(domain=domain, msgid=msgid, default='%s' % self.__doc__)
        
    def isEnabled(self):
        """ if the id cooker is enabled or not can be configured in the PropertyManager
        """
        return self.idcooker_enabled

    def _cleanId(self, text):
        """remove all charcaters not allowed in Zope ids"""
        putils = getToolByName(self, 'plone_utils')
        text = putils.normalizeString(text, relaxed=True)
        # 'relaxed' is a bit too relaxed, not removing spaces.
        text = text.replace(' ', '')
        return text

    def _object2ref(self, ref):
        """ If the passed on argument 'ref' is a BibRef Item (object), translate the import fields (authors, publication_year, title)
            to a ref dictionary (as in the parsers' code).
        """
        if type(ref) != type({}):
            obj = ref
            bib_tool = getToolByName(self, 'portal_bibliography')
            ref = bib_tool.getEntryDict(obj)
            ref['UID'] = obj.UID()

        return ref

    def _refHasAuthorNames(self, ref):
    
        return ref.has_key('authors') and (len([ author['lastname'] for author in ref['authors'] if author.get('lastname', None) ]) >= 1)

    def _cookIdCore(self, ref, new_id='', **kwargs):
        """ the core id cooking method that is overridden in the individual id cookers
        """
        return new_id   # needs to be provided by the individual id cookers

    def getCookedBibRefId(self, ref, use_pid_on_import=True, **kwargs):
        """
        cook id for ref dict or object, ref dict / object may be a single reference only
        """
        isReferenceObject = (type(ref) != type({}))
        if isReferenceObject:
            ref = self._object2ref(ref)
        new_id = 'nobody1000'
        if use_pid_on_import and ref.has_key('pid') and ref['pid']:
            new_id = ref['pid']
        else:
            new_id = self._cookIdCore(ref, new_id=new_id)

        return _encode(_decode(self._cleanId(new_id)))


InitializeClass(BibliographyIdCooker)


class IdCookerFolder(Folder):
    """
    A folder that only offers to add objects that implement the
    IBibliographyIdCooker interface. 
    """
    meta_type = 'IdCooker Folder'

    id = 'IdCookers'
    title = "BibliographyTool's id cooker folder"

    # we don't want 'View'
    manage_options = ( Folder.manage_options[0], ) \
                     + Folder.manage_options[2:]
    index_html = None

    def __init__(self, id, title=''):
        """
        minimal initialization
        """
        self.id = id
        # self.title = title
    
    def all_meta_types(self):
        product_infos = Products.meta_types
        possibles = []
        for p in product_infos:
            try:
                if IBibliographyIdCooker in p.get('interfaces', []):
                    possibles.append(p)
            except TypeError:
                pass
        definites = map(lambda x: x.meta_type, self.objectValues())
        return filter(lambda x,y=definites: x['name'] not in y, possibles)

InitializeClass(IdCookerFolder) 
   



##################################################
#                                                #
#    Copyright (C), 2004, Raphael Ritz           #
#    <r.ritz@biologie.hu-berlin.de>              #
#                                                #
#    Humboldt University Berlin                  #
#                                                #
##################################################

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup

from Products.CMFBibliographyAT.tool.parsers.ris import \
     RISParser

class TestRISParser(PloneTestCase.PloneTestCase):
    '''Test the RIS parser'''

    # the individual tests

    def test_Parser(self):
        source = open(setup.RIS_SOURCE, 'r').read()
        p = RISParser()
        # We need to wrap the parser in an acquisition context because it
        # actually looks up behaviour with getToolByName
        p = p.__of__(self.portal.portal_bibliography)
        entries = p.getEntries(source)
        #raise Exception(entries)
        self.failUnless( len(entries) == 1 )
        self.failUnless( entries[0]['title'] == 'Markets and Municipalities: A Study of the Behavior of the Danish Municipalities' )
        self.failUnless( entries[0]['pages'] == '79--102' )
        self.failUnless( len( entries[0]['authors'] ) == 2 )
        self.failUnless( entries[0]['authors'][0]['lastname'] == 'Christoffersen' )
        self.failUnless( entries[0]['authors'][0]['firstname'] == 'Henrik' )
        self.failUnless( entries[0]['authors'][0]['middlename'] == '' )
        self.failUnless( entries[0]['authors'][1]['lastname'] == 'Paldam' )
        self.failUnless( entries[0]['authors'][1]['firstname'] == 'Martin' )
        self.failUnless( entries[0]['authors'][1]['middlename'] == '' )
        self.failUnless( entries[0]['volume'] == '114' )
        self.failUnless( entries[0]['number'] == '1-2' )
        self.failUnless( entries[0]['publication_year'] == '2003' )
        self.failUnless( entries[0]['journal'] == 'Public Choice' )
        # XXX This test could be adjusted if the parser got smarter about
        #     about converting 'Mar.' into 'March'
        self.failUnless( entries[0]['publication_month'] == '01' )

    # end of the individual tests

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestRISParser))
    return suite

if __name__ == '__main__':
    framework()



## Controller Python Script "bibliography_entry_addOwnerToLocalRoles"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=memberId=None
##title=
##

from Products.CMFCore.permissions import ModifyPortalContent

bib_tool = context.portal_bibliography
mtool = context.portal_membership

if memberId is not None:
    
    # only applicable to bibliography reference items and if authenticated member is owner of item!!!
    if context.portal_type in bib_tool.getReferenceTypes() and mtool.checkPermission(ModifyPortalContent, context):

	context.manage_addLocalRoles(memberId, ('Owner',))


## Script (Python) "DefaultSource"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
return context.meta_type[:-9] + "." # expecting something like '...Reference'



from Products.CMFCore.CMFCorePermissions import AddPortalContent
from Products.Archetypes.public import DisplayList

ADD_CONTENT_PERMISSION = AddPortalContent
PROJECTNAME = "bibTypes"
SKINS_DIR = 'skins'

GLOBALS = globals()

MONTH = DisplayList((
    ('', 'No month available'),
    ('Jan', 'January'),
    ('Feb', 'February'),
    ('Mar', 'March'),
    ('Apr', 'April'),
    ('May', 'May'),
    ('Jun', 'June'),
    ('Jul', 'July'),
    ('Aug', 'August'),
    ('Sep', 'September'),
    ('Oct', 'October'),
    ('Nov', 'November'),
    ('Dec', 'December'),
    ))


############################################################################
#                                                                          #
#             copyright (c) 2004 ITB, Humboldt-University Berlin           #
#             written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                          #
############################################################################

"""PDFRenderer class"""

# Python stuff
import os, tempfile, shutil, types

# Zope stuff
from Globals import InitializeClass
from App.Dialogs import MessageDialog
from OFS.PropertyManager import PropertyManager

# CMF stuff
from Products.CMFCore.utils import getToolByName
from bibtex import _entity_mapping

from Products.CMFBibliographyAT.encodings import _utf8enc2latex_mapping

from Products.CMFBibliographyAT.utils import _encode, _decode

# Bibliography stuff
from Products.CMFBibliographyAT.tool.renderers.base \
     import IBibliographyRenderer, BibliographyRenderer

DEFAULT_TEMPLATE = r"""
\documentclass[english,12pt,a4paper]{article}
\usepackage[T1]{fontenc}
\usepackage[latin9]{inputenc}
\usepackage{bibmods}
\usepackage{bibnames}
\usepackage{showtags}
\renewcommand{\refname}{~}

\begin{document}
\begin{center}
{\large \bf %s}\\
(URL: %s)
\end{center}

~\hfill \today

\nocite{*}
\bibliographystyle{abbrv}
\bibliography{references}

\end{document}
"""

class PDFRenderer(BibliographyRenderer):
    """
    A renderer that produces a PDF document containing bibliographical references. The PDF export is rendered by PDFLaTeX.
    """

    __implements__ = (IBibliographyRenderer ,)

    meta_type = "PDF Renderer"

    format = {'name':'PDF',
              'extension':'pdf'}
    template = ''

    _properties = tuple([ prop for prop in BibliographyRenderer._properties
                          if prop['id'] != 'default_output_encoding' ]) + \
    (
            {'id':'template', 
             'type':'text', 
             'mode':'w'},
    )

    def __init__(self,
                 id = 'pdf',
                 title = "PDF renderer (via latex/bibtex)"):
        """
        initializes id, title and the default LaTeX template
        """
        self.id = id
        self.title = title
        self.template = DEFAULT_TEMPLATE

    def isAvailable(self):
        """ test the availability of the following commands on the server's filesystem:
                
                latex, pdflatex, bibtex
                
        """ 
        # still TODO
        return True
        
    def initializeDefaultOutputEncoding(self, portal_instance=None):
        """ not needed for PDF renderer
        """        
        pass
        
    def getDefaultOutputEncoding(self):
        """ not needed for PDF renderer
        """        
        return None
        
    def getAvailableEncodings(self):
        """ not needed for PDF renderer
        """        
        return []
        
    def _normalize(self, text):
        text = self._resolveEntities(text)
        text = self._resolveUnicode(text)
        text = self._escapeSpecialCharacters(text)
        return text
				    
    def _resolveEntities(self, text):
        for entity in _entity_mapping.keys():
            text = text.replace(entity, _entity_mapping[entity])
        return text
							    
    def _resolveUnicode(self, text):
        for unichar in _utf8enc2latex_mapping.keys():
            text = _encode(_decode(text).replace(unichar, _utf8enc2latex_mapping[unichar]))
        return text
	    
    def _escapeSpecialCharacters(self, text):
        """
        latex escaping some (not all) special characters
        """
        escape = ['~', '#', '&', '%', '_']
        for c in escape:
            text = text.replace(c, '\\' + c )
        return text
																				    	
    def render(self, objects, **kwargs):
        """
        renders a bibliography object in PDF format
        """
        if type(objects) not in (types.ListType, types.TupleType):
            objects =[objects]
        bib_tool = getToolByName(objects[0], 'portal_bibliography')
        ref_types = bib_tool.getReferenceTypes()
        bibtex_source = ''
        container_object = objects[0]     
        for object in objects:
            bibtex_source = bibtex_source + bib_tool.render(object,
                                        		    format='BibTex',
                                        		    output_encoding='latin-1',
							    title_force_uppercase=True)
	
        return self.processSource(bibtex_source,
                                  container_object.title_or_id(),
                                  container_object.absolute_url())
        
    def processSource(self, source, title, url):
        """
        use latex/bibtex/pdflatex to generate the pdf
        from the passed in BibTeX file in 'source' using
        the (LaTeX) source tempalte from the renderer's
        'template' property
        """
        template = self.getProperty('template', DEFAULT_TEMPLATE)
        template = template % (unicode(self._normalize(title), 'utf-8').encode('latin-1'), unicode(self._normalize(url), 'utf-8').encode('latin-1'))
        wd = self.getWorkingDirectory()
        tex_path = os.path.join(wd, 'template.tex')
        bib_path = os.path.join(wd, 'references.bib')
        tex_file = open(tex_path, 'w')
        bib_file = open(bib_path, 'w')
        tex_file.write(template)
        bib_file.write(source)
        tex_file.close()
        bib_file.close()
        os.system("cd %s; latex %s"% (wd, tex_path))
        os.system("cd %s; bibtex %s"% (wd, 'template'))
        os.system("cd %s; latex %s"% (wd, 'template.tex'))
        os.system("cd %s; pdflatex %s"% (wd, tex_path))
        pdf_file= open(os.path.join(wd, "template.pdf"), 'r')
        pdf = pdf_file.read()
        pdf_file.close()
        self.clearWorkingDirectory(wd)
        return pdf


    def getWorkingDirectory(self):
        """
        returns the full path to a newly created
        temporary working directory
        """
        tmp_dir = tempfile.mkdtemp()
        renderer_dir = '/'.join(os.path.split(__file__)[:-1])
        resource_dir = os.path.join(renderer_dir, 'latex_resources')
        for file in os.listdir(resource_dir):
            source = os.path.join(resource_dir, file)
            destination = os.path.join(tmp_dir, file)
            if os.path.isfile(source):
                shutil.copy(source, destination)
        return tmp_dir

    def clearWorkingDirectory(self, wd):
        """
        removes the temporary working directory
        """
        for file in os.listdir(wd):
            try:
                path = os.path.join(wd, file)
                os.remove(path)
            except OSError:
                pass
        os.rmdir(wd)

InitializeClass(PDFRenderer)


def manage_addPDFRenderer(self, REQUEST=None):
    """ """
    ## self._setObject('pdf', PDFRenderer())
    try:
        self._setObject('pdf', PDFRenderer())
    except:
        return MessageDialog(
            title='Bibliography tool warning message',
            message='The renderer you attempted to add already exists.',
            action='manage_main')
    return self.manage_main(self, REQUEST)



##################################################
#                                                #
#    Copyright (C), 2004, Raphael Ritz           #
#    <r.ritz@biologie.hu-berlin.de>              #
#                                                #
#    Humboldt University Berlin                  #
#                                                #
##################################################

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.CMFPlone.tests import PloneTestCase
from Products.CMFBibliographyAT.tests import setup

class TestReferenceTypes(PloneTestCase.PloneTestCase):
    '''Test the reference types'''

    def afterSetUp(self):
        pass

    # some utility methods

    def getEmptyBibFolder(self):
        uf = self.folder
        uf.invokeFactory(type_name = "BibliographyFolder",
                         id = 'bib_folder')
        return getattr(uf, 'bib_folder')

    def getPopulatedBibFolder(self):
        bf = self.getEmptyBibFolder()
        med_source = open(setup.MEDLINE_TEST_MED, 'r').read()
        bf.processImport(med_source, 'medline_test.med')
        return bf   

    # the individual tests

    def test_ArticleCreation(self):
        bf = self.getEmptyBibFolder()
        bf.invokeFactory(type_name = 'ArticleReference',
                         id = 'test_article')
        self.failUnless('test_article' in bf.contentIds())

    def test_ArticleSource(self):
        bf = self.getPopulatedBibFolder()
        source = bf.CokeEtAl2003.Source()
        expected_source = 'Am J Vet Res, 64(2):225-8.'
        self.assertEqual(source, expected_source)

    def test_getPdfFolder(self):
        bf = self.getPopulatedBibFolder()
        entry = bf.CokeEtAl2003
        o1 = bf.getPdfFolder()
        o2 = entry.getPdfFolder()
        self.assertEqual(o1, o2)
        self.failUnless('pdfs' in bf.contentIds())
        folder_type = o1.portal_type
        self.assertEqual(folder_type, 'PDF Folder')
        

    # end of the individual tests

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestReferenceTypes))
    return suite

if __name__ == '__main__':
    framework()
